/* testcase.c
 * Author:  Rick Goldstein
 */

#include <xview/xview.h>
#include <xview/panel.h>
#include "snafu.h"
#include "testcase.h"

static void	show_props ( Panel_item item, Event *event );
static void	apply_snafu_props ( Snafu snafu, char *name, int severity, Snafu_type type );


Attr_attribute TEST_APP_KEY;

/* TestApp "constructor" */
static TestApp	*create_test_app ( Frame parent );


void
main ( int argc, char **argv )
{
    TestApp *app;

    (void) xv_init ( XV_INIT_ARGC_PTR_ARGV, &argc, argv, NULL );

    /* we want to do it this way because you never
     * know when it'd be handy to instantiate the whole
     * app to be used by another...
     *
     * note:  no global variables other than data keys!
     */
    app = create_test_app ( XV_NULL );

    xv_main_loop ( app->frame );
    exit ( 0 );
} 


static TestApp	*
create_test_app ( Frame parent )
{
    TestApp *app = xv_alloc ( TestApp );

    /* this key is specific to this layer of objects */
    TEST_APP_KEY = xv_unique_key();

    /* 
     * Note:  could have used GXV code, 'cept for the Snafu, of course.
     * also, might be nice to get some of these values from resources.
     */
    app->frame = xv_create ( parent, FRAME, 
			    XV_LABEL,	"Snafu Demo",
			    NULL );
    app->panel = xv_create ( app->frame, PANEL, 
			    PANEL_LAYOUT,	PANEL_VERTICAL,
			    NULL );

    app->snafu = xv_create ( app->frame, SNAFU, 
			    XV_LABEL,		"Snafu Demo:  Properties",
			    SNAFU_APPLY_PROC, 	apply_snafu_props,
			    SNAFU_SEVERITY,	100,
			    SNAFU_TYPE,		SNAFU_NUCLEAR,
			    XV_KEY_DATA,	TEST_APP_KEY, app,
			    NULL );

    app->button = xv_create ( app->panel, PANEL_BUTTON,
			     PANEL_LABEL_STRING,	"Properties...",
			     PANEL_NOTIFY_PROC,		show_props,
			     XV_KEY_DATA,		TEST_APP_KEY,	app,
			     NULL );

    app->name_msg = xv_create ( app->panel, PANEL_MESSAGE,
			       PANEL_LABEL_STRING,	"No Name            ",
			       NULL );
    app->severity_msg = xv_create ( app->panel, PANEL_MESSAGE,
				   PANEL_LABEL_STRING,	"No Severity",
				   NULL );
    app->type_msg = xv_create ( app->panel, PANEL_MESSAGE,
			       PANEL_LABEL_STRING,	"No Type",
			       NULL );

    window_fit ( app->panel );
    window_fit ( app->frame );

    return app;
} /* create_test_ap() */




/*
 * "Properties" button callback
 */
static void
show_props ( Panel_item item, Event *event )
{
    TestApp *app = (TestApp *)xv_get(item, XV_KEY_DATA, TEST_APP_KEY);
    xv_set ( app->snafu, XV_SHOW, TRUE, NULL );
} /* show_props() */



/* callback from snafu */
static void
apply_snafu_props ( Snafu snafu, char *name, int severity, Snafu_type type )
{
    TestApp *app = (TestApp *)xv_get(snafu, XV_KEY_DATA, TEST_APP_KEY);
    char buf[16];

    xv_set ( app->name_msg, PANEL_LABEL_STRING, name, NULL );

    (void) sprintf( buf, "Severity: %d", severity );
    xv_set ( app->severity_msg, PANEL_LABEL_STRING, buf, NULL );

    switch ( (int)type ) {
    case SNAFU_NUCLEAR:
	xv_set ( app->type_msg, PANEL_LABEL_STRING, "Nuclear", NULL );
	break;

    case SNAFU_CHEMICAL:
	xv_set ( app->type_msg, PANEL_LABEL_STRING, "Chemical", NULL );
	break;

    case SNAFU_BIOLOGICAL:
	xv_set ( app->type_msg, PANEL_LABEL_STRING, "Biological", NULL );
	break;

    default:
	xv_set ( app->type_msg, PANEL_LABEL_STRING, "Unknown", NULL );
    }

} /* apply_snafu_props() */
