/* cursor.c -- cursor demo
 * Author: Rick Goldstein from an idea by Ken Bibb
 * March 1992
 *---------------------------
 * The ugly #ifdefs were inserted by Ken so this
 * would compile on both his IPC (XView 3.0) and
 * on his SLC (XView 2.0)
 */

#include <stdio.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/canvas.h>
#include <xview/cursor.h>
#include <xview/svrimage.h>
#include <xview/cms.h>
 
/* the question mark image was generated by IconEdit
 * using the OLGUIFS description as input.  
 */
static short question_image[16] = {
    0x7C00, 0xC600, 0x0600, 0x0600, 0x0C00, 0x3000,
    0x6000, 0x6000, 0x6000, 0x1000, 0x6000, 0x6000,
    0x0000, 0x0000, 0x0000, 0x0000,
};
 
/* handle to the cursor window
 */
Attr_attribute CURSOR_WINDOW_KEY;
 


/* the notify procedure for the PANEL_CHOICE item
 * in the cursor_window.  This is what changes the cursor.
 */
void
cursor_choice_callback ( item, value, event )
    Panel_item item;
    int value;
    Event *event;
{
    Xv_Window	 cursor_win = (Xv_Window) xv_get ( item, XV_KEY_DATA,
    						   CURSOR_WINDOW_KEY );
    Xv_cursor	 	cursor = (Xv_cursor) NULL;
    Xv_cursor	 	qcursor = (Xv_cursor) NULL;
    Xv_singlecolor	fg, bg;
    Server_image 	qimage = (Server_image) NULL;
    int		 	csr_type;
    int		 	csr_mask;
    

    if ( !cursor_win )
        return;
 
    switch ( value )
    {
    case 0:                                                     /* basic */
        csr_type = OLC_BASIC_PTR;
        csr_mask = OLC_BASIC_MASK_PTR;
        break;

    case 1:                                                     /* move */
        csr_type = OLC_MOVE_PTR;
        csr_mask = OLC_MOVE_MASK_PTR;
        break;

    case 2:                                                     /* copy */
        csr_type = OLC_COPY_PTR;
        csr_mask = OLC_COPY_MASK_PTR;
        break;

    case 3:                                                     /* busy */
        csr_type = OLC_BUSY_PTR;
        csr_mask = OLC_BUSY_MASK_PTR;
        break;
 
    case 4:                                                     /* text */
#ifdef OLC_TEXT_MOVE_DRAG
        csr_type = OLC_TEXT_MOVE_DRAG;
        csr_mask = OLC_TEXT_MOVE_MOVE_MASK;
                /* why isn't it OLC_TEXT_MOVE_DRAG_MASK ? */
#endif
        break;
 
    case 5:                                                     /* panning */
#ifdef OLC_PANNING_MASK_PTR
        csr_type = OLC_PANNING_PTR;
        csr_mask = OLC_PANNING_MASK_PTR;
#endif
        break;
 
    case 6:                                                     /* target */
#ifdef OLC_NAVIGATION_LEVEL_MASK_PTR
        csr_type = OLC_NAVIGATION_LEVEL_PTR;
        csr_mask = OLC_NAVIGATION_LEVEL_MASK_PTR;
#endif
       break;

    case 7:                                                     /* ? */
        /* create a server image of the '?' */
        if ( !qimage )
            qimage = xv_create ( XV_NULL, SERVER_IMAGE,
                                XV_WIDTH,               16,
                                XV_HEIGHT,              16,
                                SERVER_IMAGE_BITS,      question_image,
                                NULL );
        if ( !cursor )
            cursor = (Xv_cursor) xv_find ( XV_NULL, CURSOR,
                                           CURSOR_IMAGE,        qimage,
                                           /* should have a mask image */
                                           NULL );
        break;
 
    case 8:                                                     /* basic */
        csr_type = OLC_BASIC_PTR;
        csr_mask = OLC_BASIC_MASK_PTR;
        break;

    } /* switch() */


    if ( value == 8 )
    {
	fg.red = 250; fg.green = 000; fg.blue = 00;
	bg.red = 255; bg.green = 255; bg.blue = 255;
    }
 
    /* Note:  cursors are xv_find-able so they can be shared */
    if ( !cursor )
#ifndef OLC_TEXT_MOVE_DRAG
	if ( value == 4 )
		cursor = xv_find( XV_NULL, CURSOR,
				 CURSOR_STRING,         "Ack",
				 NULL );
	else
#endif
		cursor = xv_find( XV_NULL, CURSOR,
				 CURSOR_SRC_CHAR,       csr_type,
				 CURSOR_MASK_CHAR,      csr_mask,
				 NULL );

    if (value == 8)
	xv_set( cursor,
		 CURSOR_FOREGROUND_COLOR, &fg,
		 CURSOR_BACKGROUND_COLOR, &bg,
		 NULL );
    xv_set ( cursor_win,
            WIN_CURSOR, cursor,
            NULL );
} /* cursor_choice_callback() */

void
main (argc, argv )
    int argc;
    char **argv;
{
    Frame               frame;
    Canvas              canvas;
    Panel               panel;
    Panel_choice_item   choice;
    Cms                 cms;
    
    (void) xv_init ( XV_INIT_ARGC_PTR_ARGV, &argc, argv, NULL );
    CURSOR_WINDOW_KEY = xv_unique_key();
 
    
    frame = (Frame) xv_create ( XV_NULL, FRAME,
                               FRAME_LABEL,     argv[0],
                               NULL );

    panel = (Panel)xv_create ( frame, PANEL, NULL);
    
    choice = xv_create ( panel, PANEL_CHOICE,
                        PANEL_LABEL_STRING,     "Cursor:",
                        PANEL_NOTIFY_PROC,      cursor_choice_callback,
                        PANEL_CHOICE_STRINGS,    
                                "Basic",
                                "Move",
                                "Copy",
                                "Busy",
                                "Text",
                                "Panning",
                                "Target",
                                "?",
				"Red",
                                NULL,
                        NULL );
    
    window_fit_height ( panel );
 

    /* 
     * change background color to show off cursor mask
     * note:  will share default colormap with panel, so
     * i don't need to re-allocate the Control colors.
     */
    cms = xv_create ( XV_NULL, CMS,
		     CMS_SIZE,		1,
                     CMS_NAMED_COLORS,  "steel blue", NULL,
                     NULL );
    
    canvas = (Canvas) xv_create ( frame, CANVAS, NULL );

    xv_set ( canvas_paint_window(canvas),
            WIN_CMS,                    cms,
            WIN_BACKGROUND_COLOR,       0,
            NULL );


    xv_set ( choice,
            XV_KEY_DATA, CURSOR_WINDOW_KEY, canvas_paint_window(canvas),
            NULL );

    window_fit ( frame );
    xv_main_loop ( frame );
    exit ( 0 );
}
 

