/* frame.multiple.c -- Multiple Base Windows
 * Author:  Kenneth Bibb
 * January 1992 -- Arastar Consulting
 *--------------------------------------
 */

#include <stdlib.h>
#include <sys/param.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/canvas.h>
#include <xview/xv_xrect.h>
#include <xview/openmenu.h>

Attr_attribute BASE_FRAME_KEY;
Attr_attribute SITE_FRAME_KEY;

void menu_stub(menu,menu_item)
Menu menu;
Menu_item menu_item;
{
	char s[64];

	sprintf(s,"Menu: [%s]\n", (char *)xv_get(menu_item, MENU_STRING));
	xv_set(xv_get(menu,XV_KEY_DATA,BASE_FRAME_KEY),FRAME_LEFT_FOOTER,s,
			NULL);
}

void sites(menu, menu_item)
Menu menu;
Menu_item menu_item;
{
	xv_set(xv_get(menu,XV_KEY_DATA,SITE_FRAME_KEY),XV_SHOW, TRUE, NULL);
}

void push_me_proc(button,event)
Panel_item button;
Event *event;
{
	char s[64];

	sprintf(s,"Button: [%s]\n", (char *)xv_get(button, PANEL_LABEL_STRING));
	xv_set(xv_get(button,XV_KEY_DATA,BASE_FRAME_KEY),FRAME_LEFT_FOOTER,s,
			NULL);
}

int main(argc, argv)
int argc;
char **argv;
{
	Frame		frame;
	Frame		site_frame;
	Canvas		canvas;
	Panel		panel;
	Panel		Psite;			/* site panel */
	Menu		m, m_undo;

	xv_init(XV_INIT_ARGC_PTR_ARGV, &argc, argv, NULL);
	SITE_FRAME_KEY = xv_unique_key();
	BASE_FRAME_KEY = xv_unique_key();

	/* create the base frame
	 */
	frame = (Frame)xv_create(NULL, FRAME,
		FRAME_LABEL,		argv[0],
		FRAME_SHOW_FOOTER,	TRUE,
		XV_X,			300,
		XV_Y,			175,
		NULL);

	/* create the other base frame
	 */
	site_frame = (Frame)xv_create(frame, FRAME,
		FRAME_LABEL,	"Sites",
		XV_X,		350,
		XV_Y,		300,
		NULL);

	panel = (Panel)xv_create(frame, PANEL, NULL);
	Psite = (Panel)xv_create(site_frame, PANEL, NULL);

	m = (Menu)xv_create(NULL, MENU,
		MENU_ACTION_ITEM,	"New...", menu_stub,
		MENU_ACTION_ITEM,	"Save...", menu_stub,
		MENUITEM_SPACE, NULL,
		MENU_ACTION_ITEM,	"Print...",menu_stub,
		MENU_ACTION_ITEM,	"Quit",exit,
		XV_KEY_DATA,		BASE_FRAME_KEY, frame,
		NULL);

	(void)xv_create(panel, PANEL_BUTTON,
		PANEL_LABEL_STRING,	"File",
		PANEL_ITEM_MENU,	m,	
		NULL);

	m = (Menu)xv_create(NULL, MENU,
		MENU_ACTION_ITEM,	"Sites...", sites,
		XV_KEY_DATA,		SITE_FRAME_KEY, site_frame,
		NULL);

	(void)xv_create(panel, PANEL_BUTTON,
		PANEL_LABEL_STRING,	"View",
		PANEL_ITEM_MENU,	m,
		NULL);
	
	m_undo = (Menu)xv_create(NULL, MENU,
		MENU_ACTION_ITEM, "Undo Last Change", menu_stub,
		MENU_ACTION_ITEM, "Undo All Changes", menu_stub,
		XV_KEY_DATA,		BASE_FRAME_KEY, frame,
		NULL);

	m = (Menu)xv_create(NULL, MENU,
		MENU_ACTION_ITEM,	"Again", menu_stub,
		MENU_PULLRIGHT_ITEM,	"Undo", m_undo,
		MENUITEM_SPACE, NULL,
		MENU_ACTION_ITEM,	"Cut", menu_stub,
		MENU_ACTION_ITEM,	"Copy", menu_stub,
		MENU_ACTION_ITEM,	"Paste", menu_stub,
		MENUITEM_SPACE, NULL,
		MENU_ACTION_ITEM,	"Select All", menu_stub,
		MENU_ACTION_ITEM,	"Clear", menu_stub,
		XV_KEY_DATA,		BASE_FRAME_KEY, frame,
		NULL);

	(void)xv_create(panel, PANEL_BUTTON,
		PANEL_LABEL_STRING,	"Edit",
		PANEL_ITEM_MENU,	m,
		NULL);

	(void)xv_create(Psite,PANEL_BUTTON,
		PANEL_LABEL_STRING,	"Push Me",
		PANEL_NOTIFY_PROC,	push_me_proc,
		XV_KEY_DATA,		BASE_FRAME_KEY, frame,
		NULL);

	window_fit(Psite);
	window_fit(site_frame);
	window_fit_height(panel);

	canvas = (Canvas)xv_create(frame, CANVAS, 
		XV_X,			0,
		XV_WIDTH,		WIN_EXTEND_TO_EDGE,
		XV_HEIGHT,		WIN_EXTEND_TO_EDGE,
		CANVAS_X_PAINT_WINDOW,	TRUE,
		NULL);

	window_fit(frame);

	/* start the main base frame and the other base frames
	 * will follow
	 */
	xv_main_loop(frame);
	return(0);
}

