/* basic.frame.w.menus.c -- Open Look basic frame with menus
 * Author:  Kenneth Bibb
 * February 1992 -- Arastar Consulting
 */

#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/canvas.h>
#include <xview/xv_xrect.h>
#include <xview/openmenu.h>

Attr_attribute	BASE_FRAME_KEY;

/* menu callback for testing
 */
void menu_stub(menu, menu_item)
Menu menu;
Menu_item menu_item;
{
	char s[64];

	sprintf(s,"Menu: [%s]\n", (char *)xv_get(menu_item, MENU_STRING));
	xv_set(xv_get(menu,XV_KEY_DATA,BASE_FRAME_KEY),FRAME_LEFT_FOOTER,
		s, NULL);
}

/* display the popup menu
 */
void show_it(window, event)
Xv_Window window;
Event *event;
{
	if((event_action(event) == ACTION_MENU) && (event_is_down(event)))
	{
		menu_show(xv_get(window, WIN_CLIENT_DATA),
			window, event, NULL);
	}
}

/* canvas repaint function
 */
void repaint(canvas, win, dpy, xwin, xrect)
Canvas canvas;
Xv_Window win;
Display *dpy;
Window xwin;
Xv_xrectlist *xrect;
{
	GC gc;
	int width, height;

	gc = DefaultGC(dpy, DefaultScreen(dpy));
	width = (int)xv_get(win, XV_WIDTH);
	height = (int)xv_get(win, XV_HEIGHT);
}

int main(argc, argv)
int argc;
char **argv;
{
	Frame		frame;
	Canvas		canvas;
	Icon		icon;
	Server_image	cimage;
	Panel		panel;
	Menu		m, m_undo;

	xv_init(XV_INIT_ARGC_PTR_ARGV, &argc, argv, NULL);
	BASE_FRAME_KEY = xv_unique_key();

	frame = (Frame)xv_create(XV_NULL, FRAME,
		FRAME_LABEL,	argv[0],
		XV_X,		300,
		XV_Y,		175,
		NULL);

	panel = (Panel)xv_create(frame, PANEL, NULL);

	/* file menu--replace menu_stubs with names of
	 *            real functions
	 */
	m = (Menu)xv_create(XV_NULL, MENU,
		MENU_ITEM,
			MENU_STRING, "New...",
			MENU_NOTIFY_PROC, menu_stub,
			NULL,
		MENU_ITEM,
			MENU_STRING, "Save...",
			MENU_NOTIFY_PROC, menu_stub,
			NULL,
		MENUITEM_SPACE, NULL,
		MENU_ITEM,
			MENU_STRING, "Print...",
			MENU_NOTIFY_PROC, menu_stub,
			NULL,
		XV_KEY_DATA, BASE_FRAME_KEY, frame,
		NULL);

	(void)xv_create(panel, PANEL_BUTTON,
		PANEL_LABEL_STRING,	"File",
		PANEL_ITEM_MENU,	m,	
		NULL);

	/* create the view menu.  since this is application
	 * specific, I put generic strings in
	 */
	m = (Menu)xv_create(XV_NULL, MENU,
		MENU_ITEM,
			MENU_STRING,	"Item 1",
			MENU_NOTIFY_PROC, menu_stub,
			NULL,
		MENU_ITEM,
			MENU_STRING,	"Item 2",
			MENU_NOTIFY_PROC, menu_stub,
			NULL,
		NULL);

	(void)xv_create(panel, PANEL_BUTTON,
		PANEL_LABEL_STRING,	"View",
		PANEL_ITEM_MENU,	m,
		NULL);
	
	/* undo sub menu
	 */
	m_undo = (Menu)xv_create(XV_NULL, MENU,
		MENU_ITEM,
			MENU_STRING, "Undo Last Change",
			MENU_NOTIFY_PROC, menu_stub,
			NULL,
		MENU_ITEM,
			MENU_STRING, "Undo All Changes",
			MENU_NOTIFY_PROC, menu_stub,
			NULL,
		NULL);

	/* edit menu
	 */
	m = (Menu)xv_create(XV_NULL, MENU,
		MENU_ITEM,
			MENU_STRING, "Again",
			MENU_NOTIFY_PROC, menu_stub,
			NULL,
		MENU_ITEM,
			MENU_STRING, "Undo",
			MENU_PULLRIGHT, m_undo,
			NULL,
		MENUITEM_SPACE, NULL,
		MENU_ITEM,
			MENU_STRING, "Cut",
			MENU_NOTIFY_PROC, menu_stub,
			NULL,
		MENU_ITEM,
			MENU_STRING, "Copy",
			MENU_NOTIFY_PROC, menu_stub,
			NULL,
		MENU_ITEM,
			MENU_STRING, "Paste",
			MENU_NOTIFY_PROC, menu_stub,
			NULL,
		MENUITEM_SPACE, NULL,
		MENU_ITEM,
			MENU_STRING, "Select All",
			MENU_NOTIFY_PROC, menu_stub,
			NULL,
		MENU_ITEM,
			MENU_STRING, "Clear",
			MENU_NOTIFY_PROC, menu_stub,
			NULL,
		NULL);

	(void)xv_create(panel, PANEL_BUTTON,
		PANEL_LABEL_STRING,	"Edit",
		PANEL_ITEM_MENU,	m,
		NULL);

	/* fit panel to created panel items
	 */
	window_fit_height(panel);

	/* create the canvas
	 */
	canvas = (Canvas)xv_create(frame, CANVAS, 
		XV_X, 			0,
		XV_WIDTH,		WIN_EXTEND_TO_EDGE,
		XV_HEIGHT,		WIN_EXTEND_TO_EDGE,
		CANVAS_REPAINT_PROC,	repaint,
		CANVAS_X_PAINT_WINDOW,	TRUE,
		NULL);

	/* fit frame around panel and canvas
	 */
	window_fit(frame);

	/* create the popup menu
	 */
	m = (Menu)xv_create(XV_NULL, MENU,
		MENU_GEN_PIN_WINDOW,	frame, "Popup Menu",
		MENU_PULLRIGHT_ITEM,	"Undo", m_undo,
		MENU_ACTION_ITEM,	"Print...", menu_stub,
		NULL);

	/* attach popup menu and event procedure to paint window
	 */
	xv_set(canvas_paint_window(canvas),
		WIN_CONSUME_EVENTS,	WIN_MOUSE_BUTTONS, NULL,
		WIN_EVENT_PROC,		show_it,
		WIN_CLIENT_DATA,	m,
		NULL);

	xv_main_loop(frame);
	return(0);
}

