/* gauge.c -- Open Look gauge
 * Author:  Kenneth Bibb
 * January 1992 -- Arastar Consulting
 *-----------------------------------
 * Sample of how to use gauge
 *
 * What should I demo?
 *
 */

#include <stdlib.h>
#include <stdio.h>		/* for getmntent() etc	*/
#include <mntent.h>		/* for getmntent() etc	*/
#include <math.h>		/* for nint()		*/
#include <sys/param.h>
#include <sys/vfs.h>
#include <errno.h>

#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/xv_xrect.h>
#include <xview/openmenu.h>
#include <xview/icon.h>
#include <xview/svrimage.h>

#define PTN_MAX  50		/* Max number of partitions */

int		first_time = FALSE;
int		max_partitions=0;
struct itimerval timer;
struct statfs   *fs[PTN_MAX];
struct mntent	*me[PTN_MAX];

Frame		frame;
Panel		panel;
Attr_attribute	FRAME_PROP_KEY;
Attr_attribute	CHOICE_UNIT_KEY;
Attr_attribute	CHOICE_TICK_KEY;
Attr_attribute	TEXT_DELAY_KEY;
void		check_disk();

struct ption
{
	Panel_item	 msg;
	Panel_item	 gauge;
	Panel_item	 val;
} ptn[PTN_MAX];

struct defaults
{
	Bool	percent;
	Bool	ticks;
	int	delay;
} defs, udefs;



void button_set(Cunit, Ctick, Tdlay)
Panel_item Cunit;
Panel_item Ctick;
Panel_item Tdlay;
{
	(void)xv_set(Cunit,
		PANEL_VALUE,	defs.percent,
		NULL);
	(void)xv_set(Ctick,
		PANEL_VALUE,	defs.ticks,
		NULL);
	(void)xv_set(Tdlay,
		PANEL_VALUE,	defs.delay,
		NULL);
}

void group_def(button,event)
Panel_item	 button;
Event		*event;
{
	Panel_item Cunit=(Panel_item)xv_get(button,XV_KEY_DATA,CHOICE_UNIT_KEY);
	Panel_item Ctick=(Panel_item)xv_get(button,XV_KEY_DATA,CHOICE_TICK_KEY);
	Panel_item Tdlay=(Panel_item)xv_get(button,XV_KEY_DATA,TEXT_DELAY_KEY);

	if(strcmp(xv_get(button,PANEL_LABEL_STRING),"Apply") == 0)
	{
		defs.percent = (Bool)xv_get(Cunit,PANEL_VALUE);
		defs.ticks   = (Bool)xv_get(Ctick,PANEL_VALUE);
		defs.delay   = (int) xv_get(Tdlay,PANEL_VALUE);
		check_disk();
	}
	else
	{
		defs.percent = udefs.percent;
		defs.ticks   = udefs.ticks;
		defs.delay   = udefs.delay;
		button_set();
	}
}

void prop_display(button,event)
Panel_item  button;
Event      *event;
{
	(void)xv_set(xv_get(button,XV_KEY_DATA,FRAME_PROP_KEY),
		XV_SHOW,	TRUE,
		NULL);
}

void my_usage(name)
char *name;
{
	printf("%s specific options\n",name);
	printf("	-b	use blocks instead of percentages\n");
	printf("	-d n	change the delay to n seconds\n");
	printf("		(range between 1 and 300 seconds\n");
	printf("	-t	show tick marks on gauges\n\n");
	xv_usage(name);
	exit(XV_OK);
}

set_defs()
{
	/* set defaults
	 */
	udefs.percent = defs.percent;
	udefs.ticks   = defs.ticks;
	udefs.delay   = defs.delay;
}

/* this reads disk information for setting the gauges.
 * this is pretty much equivalent to the df command
 */
void check_disk()
{
	FILE		*f;
	int		 i;
	int		 percentage;
	int		 used;
	static int	 hold_delay=-1;
	char		 sbuf[8];
	char		*s=sbuf;

	if((f=fopen("/etc/mtab","r")) == NULL)
	{
		printf("unable to open mount table\n");
		exit(1);
	}
	for(i=0; ; i++)
	{
		me[i] = (struct mntent *)getmntent(f);
		if(me[i] == (struct mntent *)NULL)
                       break;
                if(!(fs[i] = (struct statfs *)malloc(sizeof(struct statfs))))
                {
                        printf("unable to alloc\n");
                        exit(2);
                }
                statfs(me[i]->mnt_dir,fs[i]);
		percentage = nint(100.-(float)((float)fs[i]->f_bavail*100.
			/((float)fs[i]->f_blocks *.9)));
		used = fs[i]->f_blocks - fs[i]->f_bfree;
		if(first_time)
		{
			ptn[i].msg = (Panel_item)xv_create(panel, PANEL_MESSAGE,
				PANEL_NEXT_ROW,		-1,
				PANEL_LABEL_STRING,	me[i]->mnt_dir,
				PANEL_LABEL_BOLD,	TRUE,
				NULL);
			ptn[i].gauge = (Panel_item)xv_create(panel, PANEL_GAUGE,
				PANEL_VALUE,		(defs.percent)
							? percentage : used,
				PANEL_TICKS,		(defs.ticks)
							? 10 : 0,
				PANEL_MAX_VALUE,	(defs.percent)
							? 100 : fs[i]->f_blocks,
				NULL);
			if(defs.percent)
				sprintf(s,"%10d%%",percentage);
			else
				sprintf(s,"%10d",used);
			ptn[i].val  = (Panel_item)xv_create(panel,PANEL_MESSAGE,
				PANEL_LABEL_STRING,	s,
				PANEL_LABEL_BOLD,	TRUE,
				NULL);
			timer.it_interval.tv_sec = defs.delay;
			timer.it_interval.tv_usec = 0;
			timer.it_value.tv_sec = defs.delay;
			timer.it_value.tv_usec = 0;
		}
		else
		{
			(void)xv_set(ptn[i].gauge,
				PANEL_VALUE,		(defs.percent)
							? percentage : used,
				PANEL_TICKS,		(defs.ticks)
							? 10 : 0,
				PANEL_MAX_VALUE,	(defs.percent)
							? 100 : fs[i]->f_blocks,
				NULL);
			if(defs.percent)
				sprintf(s,"%10d%%",percentage);
			else
				sprintf(s,"%10d",used);
			(void)xv_set(ptn[i].val,
				PANEL_LABEL_STRING,	s,
				NULL);
			if(defs.delay != hold_delay)
			{
				hold_delay = defs.delay;
				timer.it_interval.tv_sec = defs.delay;
				timer.it_value.tv_sec = defs.delay;
				notify_set_itimer_func(frame, check_disk,
					ITIMER_REAL, &timer,NULL);
			}
		}
	}
        (void)endmntent(f);	/* closes file */
	max_partitions = i;
}

int main(argc, argv)
int argc;
char **argv;
{
	Frame		Fprop;
	Panel		Pprop;
	int		c;
	int		i=0;
	int		hold=-1;
	int		holdi=0;
	Bool		helpme=FALSE;
	Panel_item	pi;
	Panel_item	Cunit;
	Panel_item	Ctick;
	Panel_item	Tdlay;
	extern char *optarg;

	xv_init(XV_INIT_ARGC_PTR_ARGV, &argc, argv,
		XV_USAGE_PROC,	my_usage,
		NULL);
	FRAME_PROP_KEY  = xv_unique_key();
	CHOICE_UNIT_KEY = xv_unique_key();
	CHOICE_TICK_KEY = xv_unique_key();
	TEXT_DELAY_KEY  = xv_unique_key();

	/* set initial defaults
	 */
	defs.percent = TRUE;
	defs.ticks = FALSE;
	defs.delay = 60;

	/* check to see if any defaults are overridden
	 */
	while((c=getopt(argc, argv, "btd:")) != -1)
		switch(c)
		{
		case 'b':	defs.percent = FALSE;
				break;
		case 't':	defs.ticks = TRUE;
				break;
		case 'd':	defs.delay = atoi(optarg);
				break;
		case '?':
		case 'h':	helpme = TRUE;
		}
	if(helpme)
	{
		xv_usage();
		exit(3);
	}

	set_defs();
	

	/* create base frame and panel
	 */
	frame = (Frame)xv_create(NULL, FRAME,
		FRAME_LABEL,			argv[0],
		XV_X,				300,
		XV_Y,				175,
		FRAME_SHOW_RESIZE_CORNER,	FALSE,
		NULL);
	panel = (Panel)xv_create(frame, PANEL, NULL);
	pi = (Panel_item)xv_create(panel, PANEL_BUTTON,
		PANEL_LABEL_STRING,		"Properties...",
		PANEL_NOTIFY_PROC,		prop_display,
		NULL);

	Fprop = (Frame)xv_create(frame, FRAME_CMD,
		FRAME_LABEL,			"Properties",
		XV_X,				350,
		XV_Y,				200,
		NULL);
	(void)xv_set(pi,
		XV_KEY_DATA,			FRAME_PROP_KEY, Fprop,
		NULL);
	Pprop = (Panel)xv_get(Fprop, FRAME_CMD_PANEL, NULL);
	Cunit=(Panel_item)xv_create(Pprop, PANEL_CHOICE,
		PANEL_LABEL_STRING,	"Units:",
		PANEL_CHOICE_STRINGS,	"Use Blocks",
					"Use Percentages",
					NULL,
		NULL);
	Ctick=(Panel_item)xv_create(Pprop, PANEL_CHOICE,
		PANEL_LABEL_STRING,	"Tickmarks:",
		PANEL_NEXT_ROW,		-1,
		PANEL_CHOICE_STRINGS,	"Hide tickmarks",
					"Show tickmarks",
					NULL,
		NULL);
	Tdlay=(Panel_item)xv_create(Pprop, PANEL_NUMERIC_TEXT,
		PANEL_NEXT_ROW,		-1,
		PANEL_LABEL_STRING,	"Delay (in seconds):",
		PANEL_VALUE,		defs.delay,
		PANEL_MAX_VALUE,	300,		/* 5 minute interval */
		PANEL_MIN_VALUE,	1,		/* 1 second interval */
		NULL);
	pi = (Panel_item)xv_create(Pprop, PANEL_BUTTON,
		PANEL_NEXT_ROW,		-1,
		XV_X,			xv_get(Tdlay,XV_X)+80,
		PANEL_LABEL_STRING,	"Apply",
		PANEL_NOTIFY_PROC,	group_def,
		XV_KEY_DATA,		CHOICE_UNIT_KEY, Cunit,
		XV_KEY_DATA,		CHOICE_TICK_KEY, Ctick,
		XV_KEY_DATA,		TEXT_DELAY_KEY,  Tdlay,
		NULL);
	(void)xv_set(Pprop,
		PANEL_DEFAULT_ITEM,	pi,
		NULL);
	(void)xv_create(Pprop, PANEL_BUTTON,
		PANEL_LABEL_STRING,	"Reset",
		PANEL_NOTIFY_PROC,	group_def,
		XV_X,			xv_get(pi,XV_X)+100,
		XV_KEY_DATA,		CHOICE_UNIT_KEY, Cunit,
		XV_KEY_DATA,		CHOICE_TICK_KEY, Ctick,
		XV_KEY_DATA,		TEXT_DELAY_KEY,  Tdlay,
		NULL);
	button_set(Cunit,Ctick,Tdlay);
	window_fit(Pprop);
	window_fit(Fprop);

	first_time = TRUE;
	check_disk();
	first_time = FALSE;

	ptn[max_partitions].msg = ptn[max_partitions].gauge = (Panel_item)NULL;

        for(i=0; i<max_partitions; i++)
	{
		if(strlen(xv_get(ptn[i].msg,PANEL_LABEL_STRING)) > hold)
		{
			hold = strlen(xv_get(ptn[i].msg,PANEL_LABEL_STRING));
			holdi = i;
		}
	}
        for(i=0; i<max_partitions; i++)
	{
		(void)xv_set(ptn[i].gauge,
			XV_X,	xv_get(ptn[holdi].msg, XV_X) +
				xv_get(ptn[holdi].msg, XV_WIDTH) +
				xv_get(panel, PANEL_ITEM_X_GAP),
			NULL);

		(void)xv_set(ptn[i].val,
			XV_X,	xv_get(ptn[holdi].gauge, XV_X) +
				xv_get(ptn[holdi].gauge, XV_WIDTH) +
				xv_get(panel, PANEL_ITEM_X_GAP),
			NULL);
	}

	window_fit(panel);
	window_fit(frame);

	notify_set_itimer_func(frame, check_disk, ITIMER_REAL, &timer,NULL);
	xv_main_loop(frame);

        return(0);
}

