/* pl.c -- Panel List test program
 * Author:  Kenneth Bibb
 *--------------------------------------
 */

#include <stdio.h>				/* for file i/o           */
/* #include <stdlib.h> */
#include <sys/param.h>				/* for MAXHOSTNAMELEN     */
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/notice.h>
#include <xview/font.h>
#include <xview/cms.h>

#define LINELENGTH 1024				/* length of a site name  */
#define MATRIXLEN  64				/* max num sites in list  */

int numsites;					/* curr num sites in list */

Attr_attribute	FRAME_BASE_KEY;			/* base frame             */
Attr_attribute	PANEL_BASE_KEY;			/* base frame's panel     */
Attr_attribute	PL_KEY;				/* main panel list        */
Attr_attribute	TXT_SYS_KEY;			/* system name text item  */
Attr_attribute	FONT_FIXED_KEY;			/* "fixed" font           */
Attr_attribute	FONT_DEFLT_KEY;			/* "lucida" font          */

#define FIXED_NUM 5				/* num items in fixed group */
int fixed_group_in_list=FALSE;			/* fg in panel list?        */
int first_group=-1;				/* row of first fg item     */
char *fg[FIXED_NUM]={	"Propaganda",		/* fixed strings for insert */
			"Proponent",
			"Query",
			"Quagmire",
			"Renaissance"};




/* add a single system name to the panel list
 *	the system name is taken from the panel text item
 *	which is shown on the same panel.
 */
void add_system(item,event)
Panel_item	item;
Event		*event;
{
	Panel_item system=xv_get(item,XV_KEY_DATA,TXT_SYS_KEY);
	Panel_item list  =xv_get(item,XV_KEY_DATA,PL_KEY);
	Panel      panel =xv_get(item,XV_KEY_DATA,PANEL_BASE_KEY);

	char  sbuf[LINELENGTH];
	char *s=sbuf;
	int i;

	xv_set(xv_get(item,XV_KEY_DATA,FRAME_BASE_KEY),
		FRAME_LEFT_FOOTER, " ",
		NULL);
	if(numsites < MATRIXLEN)
	{
		++numsites;
		strcpy(s,xv_get(system,PANEL_VALUE));
		for(i=0;i<xv_get(list,PANEL_LIST_NROWS);i++)
			if(xv_get(list,PANEL_LIST_SELECTED,i))
				break;
		xv_set(list,
			PANEL_LIST_INSERT,		i,
			PANEL_LIST_STRING,		i, s,
			NULL);
	}
	else
#ifdef XV_VERSION_NUMBER
		(void)notice_prompt(panel, NULL,
#else
		(void)xv_create(item, NOTICE,
#endif
			NOTICE_FOCUS_XY,	event_x(event), event_y(event),
			NOTICE_MESSAGE_STRINGS,
				"You Can't Add Any More Systems!", NULL,
			NOTICE_BUTTON_YES,	"Darn It",
			NULL);
}

void add_group(item,event)
Panel_item	item;
Event		*event;
{
	Panel panel      =xv_get(item,XV_KEY_DATA,PANEL_BASE_KEY);
	Panel_item system=xv_get(item,XV_KEY_DATA,TXT_SYS_KEY);
	Panel_item list  =xv_get(item,XV_KEY_DATA,PL_KEY);

	char  sbuf[LINELENGTH];
	char *s=sbuf;
	int i,j;

	xv_set(xv_get(item,XV_KEY_DATA,FRAME_BASE_KEY),
		FRAME_LEFT_FOOTER, " ",
		NULL);
	if(numsites+FIXED_NUM < MATRIXLEN)
	{
		if(fixed_group_in_list)
			(void)xv_set(xv_get(item, XV_KEY_DATA, FRAME_BASE_KEY),
				FRAME_LEFT_FOOTER,
				"The fixed group was already added",
				NULL);
		else
		{
			(void)xv_set(list,
				XV_SHOW,	FALSE,
				NULL);
			numsites += FIXED_NUM;
			for(i=0;i<xv_get(list,PANEL_LIST_NROWS);i++)
				if(xv_get(list,PANEL_LIST_SELECTED,i))
					break;
			first_group = i;
			for(j=0; j<FIXED_NUM; j++)
			{
				xv_set(list,
					PANEL_LIST_INSERT,	i+j,
					PANEL_LIST_STRING,	i+j, fg[j],
#ifdef XV_VERSION_NUMBER
					PANEL_LIST_INSERT_DUPLICATE,	FALSE,
#endif
					NULL);
			}
			(void)xv_set(list,
				XV_SHOW,	TRUE,
				NULL);
			fixed_group_in_list = TRUE;
		}
	}
	else
#ifndef XV_VERSION_NUMBER
		(void)notice_prompt(panel, NULL,
#else
		(void)xv_create(item, NOTICE,
#endif
			NOTICE_FOCUS_XY,	event_x(event), event_y(event),
			NOTICE_MESSAGE_STRINGS,
				"You Can't Add Any More Systems!", NULL,
			NOTICE_BUTTON_YES,	"Darn It",
			NULL);
}

void del_system(item,event)
Panel_item	item;
Event		*event;
{
	Panel_item system=xv_get(item,XV_KEY_DATA,TXT_SYS_KEY);
	Panel_item list  =xv_get(item,XV_KEY_DATA,PL_KEY);

	int   result;
	char  sbuf[LINELENGTH];
	char *s=sbuf;
	int   i,j;

	xv_set(xv_get(item,XV_KEY_DATA,FRAME_BASE_KEY),
		FRAME_LEFT_FOOTER, " ",
		NULL);
	sprintf(s,"Do you want to delete the %s system?",
		(char *)xv_get(system,PANEL_VALUE));
#ifdef XV_VERSION_NUMBER				/*   XV 2.0 */
	result = notice_prompt(xv_get(item,XV_KEY_DATA,PANEL_BASE_KEY), NULL,
#else							/* > XV 2.0 */
	(void)xv_create(xv_get(item,XV_KEY_DATA,PANEL_BASE_KEY), NOTICE,
		NOTICE_STATUS,		&result,
#endif
		NOTICE_FOCUS_XY,	event_x(event), event_y(event),
		NOTICE_MESSAGE_STRINGS,	s, NULL,
		NOTICE_BUTTON_YES,	"Delete It",
		NOTICE_BUTTON_NO,	"Cancel",
		NULL);
	if(result==NOTICE_YES)
	{
		for(i=0;i<xv_get(list,PANEL_LIST_NROWS);i++)
			if(xv_get(list,PANEL_LIST_SELECTED,i))
				break;
		xv_set(list, PANEL_LIST_DELETE,i, NULL); /* delete it */
		--numsites;
		for(i=0;i<xv_get(list,PANEL_LIST_NROWS);i++)
			if(xv_get(list,PANEL_LIST_SELECTED,i))
				break;
		xv_set(system,
			PANEL_VALUE,xv_get(list,PANEL_LIST_STRING,i),
			NULL);
	}
}

void del_group(item,event)
Panel_item	item;
Event		*event;
{
	Panel_item list  =xv_get(item,XV_KEY_DATA,PL_KEY);
	int   result;
	char  sbuf[LINELENGTH];
	char *s=sbuf;
	int   i,j;

	xv_set(xv_get(item,XV_KEY_DATA,FRAME_BASE_KEY),
		FRAME_LEFT_FOOTER, " ",
		NULL);
	if(fixed_group_in_list)
	{
		for(i=FIXED_NUM; i; i--)
			xv_set(list, PANEL_LIST_DELETE,first_group+i-1, NULL);
		numsites -= FIXED_NUM;
		for(i=0;i<xv_get(list,PANEL_LIST_NROWS);i++)
			if(xv_get(list,PANEL_LIST_SELECTED,i))
				break;
		fixed_group_in_list = FALSE;
	}
	else
		(void)xv_set(xv_get(item, XV_KEY_DATA, FRAME_BASE_KEY),
			FRAME_LEFT_FOOTER, "The fixed group does not exist",
			NULL);
}

void list_proc(item,string,client_data,op,event,row)
Panel_item	item;
char	       *string;
caddr_t		client_data;
Panel_list_op	op;
Event		*event;
int		row;
{
	char  sbuf[LINELENGTH];
	char *s=sbuf;

	if(op == PANEL_LIST_OP_SELECT)
	{
		sprintf(s,"selected: [%d][%s]\n",row,string);
		xv_set(xv_get(item,XV_KEY_DATA,FRAME_BASE_KEY),
			FRAME_LEFT_FOOTER, s,
			NULL);
		xv_set(xv_get(item,XV_KEY_DATA,TXT_SYS_KEY),
			PANEL_VALUE,	string,
			NULL);
	}
}

void toggle_font(button,event)
Panel_item button;
Event *event;
{
	Panel_item	list    = xv_get(button,XV_KEY_DATA,PL_KEY);
	Xv_Font fixed   = (Xv_Font)xv_get(button, XV_KEY_DATA,FONT_FIXED_KEY);
	Xv_Font def     = (Xv_Font)xv_get(button, XV_KEY_DATA,FONT_DEFLT_KEY);
	int i,j;

	i = (int)xv_get(list, PANEL_LIST_FIRST_SELECTED);
	if((j=xv_get(list, PANEL_LIST_FONT, i)) == fixed)
	{
		(void)xv_set(list,
			PANEL_LIST_FONT, i, def,
			NULL);
	}
	else
	{
		(void)xv_set(list,
			PANEL_LIST_FONT, i, fixed,
			NULL);
	}
#ifdef NOT_NEEDED
	(void)xv_set(list,
		XV_SHOW, TRUE,
		NULL);
#endif
}

void toggle_all(button,event)
Panel_item button;
Event *event;
{
	Panel_item	list    = xv_get(button,XV_KEY_DATA,PL_KEY);
	Xv_Font fixed   = (Xv_Font)xv_get(button, XV_KEY_DATA,FONT_FIXED_KEY);
	Xv_Font def     = (Xv_Font)xv_get(button, XV_KEY_DATA,FONT_DEFLT_KEY);
	int i,j;

	if((j=xv_get(list, PANEL_LIST_FONT, 0)) == fixed)
	{
		printf("fixed: %d %d\n",fixed,def);
		(void)xv_set(list,
			XV_FONT, def,
			NULL);
	}
	else
	{
		printf("deflt: %d %d\n",fixed,def);
		(void)xv_set(list,
			XV_FONT, fixed,
			NULL);
	}
	(void)xv_set(list,
		XV_SHOW, TRUE,
		NULL);
}




int scmp(i,j)					/* string compare for qsort */
char **i, **j;
{
	return(strcmp(*i,*j));
}




int main(argc,argv)
int argc;
char **argv;
{
	FILE *sysfile;
	Frame frame;
	Panel panel;
	Panel_item list;
	Panel_item system;
	Panel_item pb;
	Xv_Font fixed;
	Xv_Font def_font;
	int i,j;
	int button_diff;
	int maxsite;
	char  sbuf[LINELENGTH];
	char *s=sbuf;
	char  site[LINELENGTH];
	char *sitelist[64];

	if((sysfile=fopen("/etc/uucp/Systems","r")) == NULL)
	{
		printf("%s: Unable to open Systems file\n",argv[0]);
		exit(1);
	}
	s=fgets(s,LINELENGTH,sysfile);
	for(j=0; j<MATRIXLEN && !feof(sysfile);)
	{
		if(s[0] != '#' && !isspace(s[0]))
		{
		if((sitelist[j]=(char *)malloc(LINELENGTH*sizeof(char)))==NULL)
			{
				printf("%s: out of mem (%d)\n",argv[0],j);
				exit(2);
			}
			strtok(s," 	,.()");
			strcpy(sitelist[j++],s);
		}
		s=fgets(s,LINELENGTH,sysfile);
	}
	sitelist[j]=NULL;
	numsites = j;
	fclose(sysfile);

	qsort(sitelist,numsites,sizeof(sitelist[0]),scmp);

	xv_init(XV_INIT_ARGC_PTR_ARGV,&argc,argv,NULL);
	FRAME_BASE_KEY = xv_unique_key();
	PANEL_BASE_KEY = xv_unique_key();
	PL_KEY         = xv_unique_key();
	TXT_SYS_KEY    = xv_unique_key();
	FONT_FIXED_KEY = xv_unique_key();
	FONT_DEFLT_KEY = xv_unique_key();

	frame = (Frame)xv_create(NULL,FRAME,
		FRAME_LABEL,		argv[0],
		FRAME_SHOW_FOOTER,	TRUE,
		NULL);
	panel = (Panel)xv_create(frame,PANEL,NULL);
	list = xv_create(panel,PANEL_LIST,
		PANEL_LABEL_STRING,	"Systems",
#ifdef XVIEW3
		PANEL_LIST_INSERT_DUPLICATE,	FALSE,
#endif
		PANEL_LIST_DISPLAY_ROWS,	3,
		PANEL_NOTIFY_PROC,	list_proc,
		XV_KEY_DATA,		FRAME_BASE_KEY, frame,
		NULL);
	for(i=0; i<numsites; i++)
	{
		xv_set(list,
			PANEL_LIST_INSERT,	i,
			PANEL_LIST_STRING,	i, sitelist[i],
			PANEL_LIST_CLIENT_DATA,	i, i,
			NULL);
	}
	system=xv_create(panel,PANEL_TEXT,
		PANEL_LABEL_STRING,	"System",
		PANEL_VALUE,		sitelist[0],
		PANEL_VALUE_DISPLAY_LENGTH,	11,
		PANEL_VALUE_STORED_LENGTH,	MAXHOSTNAMELEN,
		NULL);
	(void)xv_set(system,
		XV_X,	xv_get(system, XV_X) + 100,
		NULL);
	(void)xv_set(list,
		XV_KEY_DATA,		TXT_SYS_KEY, system,
		NULL);
	pb = (Panel_item)xv_create(panel,PANEL_BUTTON,
		XV_X,			xv_get(system,XV_X),
		XV_Y,			xv_get(system,XV_Y)+
					xv_get(system,XV_HEIGHT)+
					xv_get(panel,PANEL_ITEM_X_GAP),
		PANEL_LABEL_STRING,	"Add",
		PANEL_NOTIFY_PROC,	add_system,
		XV_KEY_DATA,		FRAME_BASE_KEY, frame,
		XV_KEY_DATA,		PANEL_BASE_KEY,	panel,
		XV_KEY_DATA,		PL_KEY, list,
		XV_KEY_DATA,		TXT_SYS_KEY, system,
		NULL);
	button_diff = (int)xv_get(pb,XV_Y) - (int)xv_get(system,XV_Y);
	(void)xv_create(panel, PANEL_BUTTON,
		XV_X,			xv_get(pb,XV_X),
		XV_Y,			xv_get(pb,XV_Y)+button_diff,
		PANEL_LABEL_STRING,	"Add Fixed Group",
		PANEL_NOTIFY_PROC,	add_group,
		XV_KEY_DATA,		FRAME_BASE_KEY, frame,
		XV_KEY_DATA,		PANEL_BASE_KEY,	panel,
		XV_KEY_DATA,		PL_KEY, list,
		NULL);
	(void)xv_create(panel,PANEL_BUTTON,
		XV_X,			xv_get(pb,XV_X),
		XV_Y,			xv_get(pb,XV_Y)+(button_diff*2),
		PANEL_LABEL_STRING,	"Delete",
		PANEL_NOTIFY_PROC,	del_system,
		XV_KEY_DATA,		FRAME_BASE_KEY, frame,
		XV_KEY_DATA,		PANEL_BASE_KEY,	panel,
		XV_KEY_DATA,		PL_KEY, list,
		XV_KEY_DATA,		TXT_SYS_KEY, system,
		NULL);
	(void)xv_create(panel, PANEL_BUTTON,
		XV_X,			xv_get(pb,XV_X),
		XV_Y,			xv_get(pb,XV_Y)+(button_diff*3),
		PANEL_LABEL_STRING,	"Delete Fixed Group",
		PANEL_NOTIFY_PROC,	del_group,
		XV_KEY_DATA,		FRAME_BASE_KEY, frame,
		XV_KEY_DATA,		PANEL_BASE_KEY,	panel,
		XV_KEY_DATA,		PL_KEY, list,
		NULL);
	def_font = (Xv_Font)xv_get(frame, XV_FONT);
	fixed    = (Xv_Font)xv_find(frame, FONT,
			FONT_NAME, "fixed",
			NULL);
	if(!fixed)
		printf("no fixed?!?\n");
	(void)xv_create(panel, PANEL_BUTTON,
		XV_X,			xv_get(pb,XV_X),
		XV_Y,			xv_get(pb,XV_Y)+(button_diff*4),
		PANEL_LABEL_STRING,	"Toggle Font",
		PANEL_NOTIFY_PROC,	toggle_font,
		XV_KEY_DATA,		PL_KEY, list,
		XV_KEY_DATA,		FONT_FIXED_KEY, fixed,
		XV_KEY_DATA,		FONT_DEFLT_KEY, def_font,
		NULL);
	(void)xv_create(panel, PANEL_BUTTON,
		XV_X,			xv_get(pb,XV_X),
		XV_Y,			xv_get(pb,XV_Y)+(button_diff*5),
		PANEL_LABEL_STRING,	"Toggle All",
		PANEL_NOTIFY_PROC,	toggle_all,
		XV_KEY_DATA,		PL_KEY, list,
		XV_KEY_DATA,		FONT_FIXED_KEY, fixed,
		XV_KEY_DATA,		FONT_DEFLT_KEY, def_font,
		NULL);
	window_fit(panel);
	window_fit(frame);
	xv_main_loop(frame);
	return(1);
}
