/* panelmsg.c -- panel message demo
 * Author:  Kenneth Bibb
 * April 1992 -- Arastar Consulting
 *--------------------------------------
 *
 * o Toggle a message.
 *	TGL_ITEM_KEY will hold the handle of the PANEL_MESSAGE
 *	button_toggle_proc() is where the action is
 *	pb is the handle of the button in main()
 *	tglm is the handle of the panel message in main()
 *
 * o Text item Change Bar
 *	TXT_ITEM_KEY will hold the handle of the PANEL_TEXT
 *	text_proc() is where the action is
 *	txtm is the handle of the panel message in main()
 *
 * o Choice Change Bar
 *	CH_ITEM_KEY will hold the handle of the PANEL_CHOICE
 *	choice_proc() is where the action is
 *	chm is the handle of the panel message in main()
 *
 * o Panel message callback
 *	detects SELECT up events
 *--------------------------------------
 * BUGS:  default value should not be hardcoded in production programs
 *	  Text item's change bar should clear
 */

#include <stdio.h>
#include <xview/xview.h>
#include <xview/panel.h>



Attr_attribute TGL_ITEM_KEY;			/* toggled message handle    */
Attr_attribute TXT_ITEM_KEY;			/* panel_text cbar handle    */
Attr_attribute CH_ITEM_KEY;			/* choice cbar handle        */
char *blanks = "               ";		/* used with the toggled msg */



/* callback for the "Toggle Message" button
 */
void button_toggle_proc(item, event)
Panel_item item;
Event *event;
{
	/* retrieve the handle of the panel_message which we're going
	 * to manipulate in the following xv_set()
	 */
	Panel_item tm = (Panel_item)xv_get(item,XV_KEY_DATA, TGL_ITEM_KEY);

	/* change the PANEL_LABEL_STRING on retrived panel_message.
	 * if the string is not blank, replace PANEL_LABEL_STRING with blanks
	 * if the string is blank,     replace PANEL_LABEL_STRING with message
	 */
	xv_set(tm, PANEL_LABEL_STRING,
		((strcmp((char *)xv_get(tm,PANEL_LABEL_STRING), blanks) != 0)
				? blanks : "No Message Here"),
		NULL);
}

/* callback for the "What?" text item
 */
void text_proc(item,event)
Panel_item item;
Event *event;
{
	Panel_item	tcb = xv_get(item,XV_KEY_DATA,TXT_ITEM_KEY);

	/* change the PANEL_LABEL_STRING on retrieved panel_message.
	 * if the PANEL_VALUE is not NULL, display "|"
	 * if entering value use dimmed |
	 */
	xv_set(tcb,PANEL_LABEL_STRING,(xv_get(item,PANEL_VALUE)) ? "|" : " ",
		PANEL_INACTIVE, TRUE,
		NULL);
}

/* callback for the "Who ate the cookies?" choice
 */
void choice_proc(item,value,event)
Panel_item	item;
int		value;
Event		*event;
{
	/* hardcoded default value which should be replaced
	 * with something more elegant
	 */
	static int def_choice=0;
	Panel_item	tcb = xv_get(item,XV_KEY_DATA,CH_ITEM_KEY);

	xv_set(tcb,PANEL_LABEL_STRING,((value!=def_choice) ? "|" : " "),
		NULL);
}

/* callback for the "No Message Here" panel_message
 */
void message_proc(item,event)
Panel_item	item;
Event		*event;
{
	/* beep at any stupid enough to click here ;)
	 */
	printf("\07");
	fflush(stdout);
}


main(argc, argv)
int argc;
char **argv;
{
	Frame		frame;			/* base frame            */
	Panel		panel;			/* base panel            */
	Panel_item	pb;			/* toggle button handle  */
	Panel_item	tglm;			/* toggle message handle */
	Panel_item	txtm;			/* text message handle   */
	Panel_item	chm;			/* choice message handle */
	char		mbuf[20];
	char		*message=mbuf;		/* field for panel_text  */

	xv_init(XV_INIT_ARGC_PTR_ARGV, &argc, argv, NULL);
	TXT_ITEM_KEY = xv_unique_key();		/* set the keys          */
	TGL_ITEM_KEY = xv_unique_key();
	CH_ITEM_KEY  = xv_unique_key();

	frame = (Frame)xv_create(NULL, FRAME,
		FRAME_LABEL,			argv[0],
		XV_X,				300,
		XV_Y,				175,
		FRAME_SHOW_RESIZE_CORNER,	FALSE,
		NULL);
	panel = (Panel)xv_create(frame, PANEL, NULL);


	/* set up toggle button
	 */
	pb =   (Panel_item)xv_create(panel, PANEL_BUTTON,
		PANEL_LABEL_STRING,	"Toggle Message",
		PANEL_NOTIFY_PROC,	button_toggle_proc,
		NULL);
	tglm = (Panel_item)xv_create(panel, PANEL_MESSAGE,
		PANEL_LABEL_STRING,	blanks,
		PANEL_NOTIFY_PROC,	message_proc,
		NULL);
	(void)xv_set(pb, XV_KEY_DATA, TGL_ITEM_KEY, tglm, NULL);


	/* set up text field
	 */
	txtm = (Panel_item)xv_create(panel, PANEL_MESSAGE,
		PANEL_NEXT_ROW,	-1,
		NULL);
	(void)xv_create(panel, PANEL_TEXT,
		PANEL_LABEL_STRING,		"What?",
		PANEL_VALUE_DISPLAY_LENGTH,	10,
		PANEL_VALUE_STORED_LENGTH, 	20,
		PANEL_VALUE,			message,
		XV_KEY_DATA,			TXT_ITEM_KEY, txtm,
		PANEL_NOTIFY_LEVEL,		PANEL_ALL,
		PANEL_NOTIFY_PROC,		text_proc,
		NULL);

	/* set up choice
	 */
	chm = (Panel_item)xv_create(panel, PANEL_MESSAGE,
		PANEL_NEXT_ROW,			-1,
		PANEL_LABEL_BOLD,		TRUE,
		NULL);
	(void)xv_create(panel,PANEL_CHOICE,
		PANEL_LABEL_STRING,     "Who ate the cookies?",
		PANEL_CHOICE_STRINGS,	"April",
					"Alene",
					"Allison",
					"Amber",
					"Amy",
					NULL,
		PANEL_NOTIFY_PROC,	choice_proc,
		XV_KEY_DATA,		CH_ITEM_KEY, chm,
		NULL);
	window_fit(panel);
	window_fit(frame);

	xv_main_loop(frame);
}

