/*
 *  sor.c - An example of a blocking selection requestor.
 */

#include <stdio.h>
#include <X11/Xlib.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/sel_pkg.h>

/*
 * Global object definitions.
 */
Frame		base_frame;
Textsw		textsw;

void
button_notify(menu, result)
Menu menu;
Menu_item result;
{
    static Selection_requestor sel_requestor = NULL;
    char *data;
    int length, format;

    if ( sel_requestor == NULL )
	sel_requestor = xv_create(base_frame, SELECTION_REQUESTOR, NULL);

    xv_set(sel_requestor,
	SEL_RANK, (Atom)xv_get(result, MENU_CLIENT_DATA),
	NULL);

    data = (char *)xv_get(sel_requestor, SEL_DATA, &length, &format);

    if (length == SEL_ERROR)
	xv_set(base_frame, FRAME_LEFT_FOOTER, "Could not get selection.", NULL);
    else {
	char msg[30];

	textsw_reset(textsw, 0, 0);
	textsw_insert(textsw, data, strlen(data));
	textsw_possibly_normalize(textsw, 0);
	free(data);

	sprintf(msg, "Selection length: %d", length);
	xv_set(base_frame, FRAME_LEFT_FOOTER, msg, NULL);	
    }
}

/*
 *  Create menu for rank button.
 */
Menu
sor_menu_create(owner)
Frame owner;
{
    Menu menu;
    Display *dpy;

    dpy = (Display *)xv_get(owner, XV_DISPLAY);

    menu = xv_create(XV_NULL, MENU_COMMAND_MENU,
	MENU_TITLE_ITEM,	"Selection Rank",
	MENU_ITEM,
	    MENU_STRING,	"PRIMARY",
	    MENU_CLIENT_DATA,	XA_PRIMARY,
	    NULL,
	MENU_ITEM,
	    MENU_STRING,	"SECONDARY",
	    MENU_CLIENT_DATA,	XA_SECONDARY,
	    NULL,
	MENU_ITEM,
	    MENU_STRING,	"CLIPBOARD",
	    MENU_CLIENT_DATA,	XInternAtom(dpy, "CLIPBOARD", False),
	    NULL,
	MENU_GEN_PIN_WINDOW,	owner, "Sel. Rank",
	MENU_NOTIFY_PROC,	button_notify,
	NULL);
    return menu;
}

void
setup_window()
{
    Panel panel;

    base_frame = xv_create(NULL, FRAME,
	XV_LABEL,		"Select-O-Rama",
	FRAME_SHOW_FOOTER,	TRUE,
	NULL);

    panel = xv_create(base_frame, PANEL,
	XV_WIDTH,		414,
	NULL);

    (void)xv_create(panel, PANEL_BUTTON,
	XV_X,			16,
	XV_Y, 			16,
	PANEL_LABEL_STRING,	"Rank",
	PANEL_ITEM_MENU,	sor_menu_create(base_frame),
	NULL);

    window_fit_height(panel);

    textsw = xv_create(base_frame, TEXTSW,
	XV_X,			0,
	XV_Y,			(int)xv_get(panel, XV_HEIGHT),
	XV_WIDTH,		414,
	XV_HEIGHT,		121,
	TEXTSW_BROWSING,	TRUE,
	TEXTSW_DISABLE_LOAD,	TRUE,
	TEXTSW_IGNORE_LIMIT,	TEXTSW_INFINITY,
	NULL);

    window_fit(base_frame);
}

main(argc, argv)
int	argc;
char	**argv;
{
    xv_init(XV_INIT_ARGC_PTR_ARGV, &argc, argv, NULL);

    setup_window();
    
    xv_main_loop(base_frame);
    exit(0);
}
