/*
 *  stopwatch.c - Example of a program that takes ownership of the clipboard
 *		  selection.
 */

#include <stdio.h>
#include <time.h>
#include <X11/Xlib.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/sel_pkg.h>

/*
 *  Global object definitions.
 */
Frame		base_frame;
Display *dpy;

char clip_buf[26];
int clip_len;

/*
 *  Respond to a request for the contents of the CLIPBOARD selection
 */
static int
clip_conv_proc(owner, type, buf, length, format)
Selection_owner owner;
Atom *type;
Xv_opaque *buf;
unsigned long *length;
int *format;
{
    Atom TARGETS, LENGTH;
    static Atom targs[3] = { NULL, NULL, NULL };

    /*
     *  Initialize list of atoms we will respond to
     */
    if (targs[0] == NULL ) {
	TARGETS = XInternAtom(dpy, "TARGETS", False);
	LENGTH = XInternAtom(dpy, "LENGTH", False);

	targs[0] = TARGETS;
	targs[1] = LENGTH;
	targs[2] = XA_STRING;
    }

    if ( *type == TARGETS ) {
	*format = 32;
	*length = 3;
	*type = XA_ATOM;
	*buf = (Xv_opaque)targs;
	return TRUE;
    }
    else if ( *type == LENGTH ) {
	*format = 32;
	*length = 1;
	*type = XA_INTEGER;
	*buf = (Xv_opaque)&clip_len;
 	return TRUE;
    }
    else if ( *type == XA_STRING ) {
	*format = 8;
	*length = clip_len;
	*buf = (Xv_opaque)clip_buf;
	return TRUE;
    }
    else
    	return FALSE;
}

/*
 *  Handle loss of the selection
 */
void
sel_lose_proc(sel)
Selection_owner	sel;
{
    xv_set(base_frame, FRAME_LEFT_FOOTER, "Clipboard relinquished.", NULL);

    /*
     *  If you dynamically allocated an area to hold clipboard contents,
     *  where you'd free it.
     */
}

/*
 *  Become the owner of the CLIPBOARD selection
 */
void
own_clipboard()
{
    static Selection_owner sel_owner = NULL;

    if (sel_owner == NULL)
	sel_owner = xv_create(base_frame, SELECTION_OWNER,
	    SEL_CONVERT_PROC,	clip_conv_proc,
	    SEL_LOSE_PROC,	sel_lose_proc,
	    NULL);

    xv_set(sel_owner,
	SEL_OWN,	TRUE,
	SEL_RANK_NAME,	"CLIPBOARD",
	NULL);

    xv_set(base_frame, FRAME_LEFT_FOOTER, "Acquired clipboard.", NULL);
}

/*
 * Notify callback function for `button_item'.
 */
void
button_notify(item, event)
Panel_item	item;
Event		*event;
{
    struct timeval t;

    (void)gettimeofday(&t, NULL);
    strcpy(clip_buf, ctime(&t.tv_sec));

    clip_len = strlen(clip_buf);

    own_clipboard();
}

void
setup_window()
{
    Panel panel;

    base_frame = xv_create(NULL, FRAME,
	XV_LABEL,		"Stopwatch",
	FRAME_SHOW_FOOTER,	TRUE,
	NULL);

    panel = xv_create(base_frame, PANEL,
	WIN_BORDER, TRUE,
	NULL);

    (void)xv_create(panel, PANEL_BUTTON,
	XV_X,			16,
	XV_Y, 			16,
	PANEL_LABEL_STRING,	"Copy Time to Clipboard",
	PANEL_NOTIFY_PROC,	button_notify,
	NULL);

    window_fit(panel);
    window_fit(base_frame);
}

main(argc, argv)
int	argc;
char	**argv;
{
    xv_init(XV_INIT_ARGC_PTR_ARGV, &argc, argv, NULL);
    
    setup_window();

    dpy = (Display *)xv_get(base_frame, XV_DISPLAY);
    
    xv_main_loop(base_frame);
    exit(0);
}
