/*
 * addu.c - Notify and event callback function stubs.
 */

#include <stdio.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/font.h>
#include "addu_ui.h"


/*
 * Global object definitions.
 */
addu_window1_objects	*Addu_window1;
addu_props_win_objects	*Addu_props_win;
addu_gid_win_objects	*Addu_gid_win;

int lowest_uid, highest_uid, default_gid, logid_rule;
char homedir_dir [MAXPATHLEN - L_cuserid + 1];

/*
 * Instance XV_KEY_DATA key.  An instance is a set of related
 * user interface objects.  A pointer to an object's instance
 * is stored under this key in every object.  This must be a
 * global variable.
 */
Attr_attribute	INSTANCE;

void gid_pane_event();

/*
 *  Set reasonable default property values.
 */
void
set_props()
{
    char hostname[MAXHOSTNAMELEN + 1];

    lowest_uid = 1000;
    highest_uid = 65533;

    default_gid = 20;

    logid_rule = 0;

    gethostname(hostname, MAXHOSTNAMELEN + 1);
    sprintf(homedir_dir, "/home/%s", hostname);
}

void
init_window()
{
    int id;
    char ids[10];

    xv_set(Addu_window1->fname_item, PANEL_VALUE, "", NULL);
    xv_set(Addu_window1->logid_item, PANEL_VALUE, "", NULL);

    id = findopen(lowest_uid, highest_uid);
    if ( id == -1 ) {
	xv_set(Addu_window1->window1, FRAME_LEFT_FOOTER,
	    "No available UID in the range you specified.", NULL);
	window_bell(Addu_window1->window1);
    }
    else {
	sprintf(ids, "%d", id);
	xv_set(Addu_window1->uid_field_item, PANEL_VALUE, ids, NULL);
    }

    sprintf(ids, "%d", default_gid);
    xv_set(Addu_window1->gid_field_item, PANEL_VALUE, ids, NULL);

    xv_set(Addu_window1->home_dir_item, PANEL_VALUE, "", NULL);
    xv_set(Addu_window1->main_lower_pane, PANEL_CARET_ITEM,
	Addu_window1->fname_item, NULL);
}

main(argc, argv)
	int	argc;
	char	**argv;
{
    /*
     *  This may being run in setuid mode, so we'll conditionally lower
     *  our privilege level.
     */
   setprivs(FALSE);

    /*
     * Initialize XView.
     */
    xv_init(XV_INIT_ARGC_PTR_ARGV, &argc, argv, NULL);
    INSTANCE = xv_unique_key();
    
    /*
     * Initialize user interface components.
     * Do NOT edit the object initializations by hand.
     */
    Addu_window1 = addu_window1_objects_initialize(NULL, NULL);
    Addu_gid_win = addu_gid_win_objects_initialize(NULL, Addu_window1->window1);
    Addu_props_win = addu_props_win_objects_initialize(NULL, Addu_window1->window1);

    xv_set(Addu_gid_win->gid_pane,
	WIN_EVENT_PROC,	gid_pane_event,
	NULL);

    set_props();

    init_window();

    /*
     * Turn control over to XView.
     */
    xv_main_loop(Addu_window1->window1);
    exit(0);
}

/*
 * User-defined action for `shell_menu'.
 */
void
set_shell(menu, menu_item)
Menu menu;
Menu_item menu_item;
{
    xv_set(Addu_window1->shell_field_item,
	PANEL_VALUE, (char *)xv_get(menu_item, MENU_STRING),
	NULL);
}

/*
 * Menu handler for `shell_menu'.
 */
Menu
shell_menu_notify(menu, op)
Menu		menu;
Menu_generate	op;
{
    static int made_menu = FALSE;

    if (op == MENU_DISPLAY) {
	if (!made_menu) {
	    struct stat statbuf;
	    int i, nitems;

	    if (stat("/etc/shells", &statbuf) == 0) {
		FILE *sfile;

		if ((sfile = fopen("/etc/shells", "r")) != (FILE *)NULL) {
		    char buf[100];

		    nitems = (int)xv_get(menu, MENU_NITEMS);
		    for ( i = nitems; i > 1; i--)
			xv_set(menu, MENU_REMOVE, i, NULL);

		    while( fgets(buf, 100, sfile) != (char *)NULL) {
			buf[strlen(buf) - 1] = '\0';
			xv_set(menu, MENU_APPEND_ITEM,
			    (Menu_item)xv_create(NULL, MENUITEM,
				MENU_STRING,	strdup(buf),
				MENU_NOTIFY_PROC, set_shell,
				NULL),
			    NULL);
		    }
		    fclose(sfile);
		}
	    }
	    else {
		for ( i = nitems; i > 1; i--)
		    xv_set((Menu_item)xv_get(menu, MENU_NTH_ITEM, i),
			MENU_NOTIFY_PROC, set_shell, NULL);
	    }

	    made_menu = TRUE;
	}
    }
    return menu;
}

void
prep_props_window()
{
    char ids[10];

    xv_set(Addu_props_win->range_from_item, PANEL_VALUE, lowest_uid, NULL);
    xv_set(Addu_props_win->range_to_item, PANEL_VALUE, highest_uid, NULL);
    xv_set(Addu_props_win->props_home_item, PANEL_VALUE, homedir_dir, NULL);

    sprintf(ids, "%d", default_gid);
    xv_set(Addu_props_win->default_gid_item, PANEL_VALUE, ids, NULL);
}

/*
 * Notify callback function for `props_item'.
 */
void
addu_window1_props_item_notify_callback(item, event)
Panel_item	item;
Event		*event;
{
    prep_props_window();

    xv_set(Addu_props_win->props_win, XV_SHOW, TRUE, NULL);
}

/*
 * Notify callback function for `gid_item'.
 */
void
addu_window1_gid_item_notify_callback(item, event)
Panel_item	item;
Event		*event;
{
    static int got_groups = FALSE;

    if (!got_groups) {
	set_gidlist(Addu_gid_win->gid_list);
	got_groups = TRUE;
    }
	
    xv_set(Addu_gid_win->gid_win, FRAME_CMD_PUSHPIN_IN, TRUE, NULL);
    xv_set(Addu_gid_win->gid_win, XV_SHOW, TRUE, NULL);
}

/*
 * Notify callback function for `gid_list'.
 */
int
gid_list_notify(item, string, client_data, op, event, row)
Panel_item	item;
char		*string;
Xv_opaque	client_data;
Panel_list_op	op;
Event		*event;
int		row;
{
    if (op == PANEL_LIST_OP_SELECT) {
	char gid[7];

	sprintf(gid, "%d", (int)client_data);
        xv_set(Addu_window1->gid_field_item, PANEL_VALUE, gid, NULL);
    }

    return XV_OK;
}

/*
 * Notify callback function for `uid_button_item'.
 */
void
uid_notify(item, event)
Panel_item	item;
Event		*event;
{
}


/*
 * Notify callback function for `fname_item'.
 */
Panel_setting
fname_notify(item, event)
Panel_item	item;
Event		*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    char first[L_cuserid], mid[L_cuserid], last[L_cuserid], logid[L_cuserid],
	*p;
    int numnames;

    numnames = sscanf(value, "%s %s %s", first, mid, last);
    if (numnames == 2) strcpy(last, mid);

    switch(logid_rule) {
    /*
     *  Jane Doe -> jdoe
     */
    case 0:  
	sprintf(logid, "%1.1s%1.7s", first, last);
	break;
    /*
     *  Jane Doe -> jane
     */
    case 1:
	strncpy(logid, first, L_cuserid);
	logid[L_cuserid - 1] = '\0';
	break;
    /*
     *  Jane Doe -> janed
     */
    case 2:
	sprintf(logid, "%1.7s%1.1s", first, last);
	break;
    /*
     *  Jane Doe -> doej
     */
    case 3:
	sprintf(logid, "%1.7s%1.1s", last, first);
	break;
    /*
     *  Jane Doe -> doe
     */
    case 4:
	strncpy(logid, last, L_cuserid);
	logid[L_cuserid - 1] = '\0';
	break;
    /*
     *  Jane Q. Doe -> jqd
     */
    case 5:
	logid[0] = first[0];
	if (numnames == 2) {
	    logid[1] = last[0];
	    logid[2] = '\0';
	}
	else {
	    logid[1] = mid[0];
	    logid[2] = last[0];
	    logid[3] = '\0';
	}
	break;
    }

    p = logid - 1;
    while ( *(++p) != '\0') if ( isupper(*p) ) *p = tolower(*p);
    xv_set(Addu_window1->logid_item, PANEL_VALUE, logid, NULL);

    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `logid_item'.
 */
Panel_setting
logid_notify(item, event)
Panel_item	item;
Event		*event;
{
    char *	value = (char *) xv_get(item, PANEL_VALUE);
    char dir[MAXPATHLEN];

    sprintf(dir, "%s/%s", homedir_dir, value);
    xv_set(Addu_window1->home_dir_item, PANEL_VALUE, dir, NULL);

    return panel_text_notify(item, event);
}

/*
 * Notify callback function for `apply_item'.
 */
void
apply_notify(item, event)
Panel_item	item;
Event		*event;
{
    char cmd[200];

    sprintf(cmd, "/usr/etc/install/add_user %s %d %d \"%s\" %s %s",
	(char *)xv_get(Addu_window1->logid_item, PANEL_VALUE),
	atoi((char *)xv_get(Addu_window1->uid_field_item, PANEL_VALUE)),
	atoi((char *)xv_get(Addu_window1->gid_field_item, PANEL_VALUE)),
	(char *)xv_get(Addu_window1->fname_item, PANEL_VALUE),
	(char *)xv_get(Addu_window1->home_dir_item, PANEL_VALUE),
	(char *)xv_get(Addu_window1->shell_field_item, PANEL_VALUE));

/*    system(cmd); */
    fprintf(stderr, "%s\n", cmd);

    init_window();
}

/*
 * Notify callback function for `props_apply_item'.
 */
void
props_apply_notify(item, event)
Panel_item	item;
Event		*event;
{
    logid_rule = (int)xv_get(Addu_props_win->logid_rule_item, PANEL_VALUE);
    lowest_uid = (int)xv_get(Addu_props_win->range_from_item, PANEL_VALUE);
    highest_uid = (int)xv_get(Addu_props_win->range_to_item, PANEL_VALUE);
    strcpy(homedir_dir, (char *)xv_get(Addu_props_win->props_home_item,
	PANEL_VALUE));
    default_gid = atoi((char *)xv_get(Addu_props_win->default_gid_item,
	PANEL_VALUE));
}


/*
 * Notify callback function for `reset_item'.
 */
void
reset_notify(item, event)
Panel_item	item;
Event		*event;
{
    init_window();
}

/*
 * Notify callback function for `props_reset_item'.
 */
void
props_reset_notify(item, event)
Panel_item	item;
Event		*event;
{
    prep_props_window();
}

/*
 *  Monitors resize events on the GID chooser panel
 */
void
gid_pane_event(window, event)
Xv_window window;
Event	*event;
{
    static int wgrid = -1;

    /*
     *  Check for a resize event
     */
    if (event_action(event) == WIN_RESIZE) {
	int rh, nrows, nlist_h, xtra;

	if ( wgrid == -1) {
	    Xv_Font	font;
	    Window_rescale_state scale;

	    /*
	     *  What scale is the window at?
	     */
	    font = (Xv_Font)xv_get(Addu_window1->window1, XV_FONT);
	    scale = (Window_rescale_state)xv_get(font, FONT_SCALE);
	    switch(scale) {
	    case WIN_SCALE_SMALL:
		wgrid = 8;
		break;   
	    case WIN_SCALE_LARGE:
		wgrid = 12;
		break;
	    case WIN_SCALE_EXTRALARGE:
		wgrid = 16;
		break;
	    default:
		wgrid = 10;
	    }
	}

	/*
	 *  "xtra" is the part of the scrolling list height that is not
	 *  contributed by the rows themselves
	 */
	rh = (int)xv_get(Addu_gid_win->gid_list, PANEL_LIST_ROW_HEIGHT);
	xtra = (int)xv_get(Addu_gid_win->gid_list, XV_HEIGHT) -
	    rh * (int)xv_get(Addu_gid_win->gid_list, PANEL_LIST_DISPLAY_ROWS);

	/*
	 *  Figure out how long the scrolling list should be now
	 */
	nlist_h = (int)xv_get(Addu_gid_win->gid_pane, XV_HEIGHT) -
	    (int)xv_get(Addu_gid_win->gid_list, XV_Y) - wgrid * 2;

	nrows = (nlist_h - xtra) / rh;
	nlist_h = nrows * rh  + xtra;

	xv_set(Addu_gid_win->gid_list,
	    XV_HEIGHT,			nlist_h,
	    PANEL_LIST_DISPLAY_ROWS,	nrows,
	    NULL);
    }
}
