/*
 * find.c - routines supporting the scanmsg find window.
 */

#include <stdio.h>
#include <sys/param.h>
#include <sys/types.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include "scanwin_ui.h"
#include "msgboard.h"

/*
 * Global object definitions.
 */
extern scanwin_frame_objects		*scan_ip;
extern scanwin_find_frame_objects	*find_ip;

extern int cur_msg, num_msgs, num_selected;

/*
 *  Returns TRUE if string is in lowercased version of line's panel_value.
 *  Convert string to lowercase before calling this routine.
 */
int
scan_find(line, string, clear_msgs)
int line, clear_msgs;
char *string;
{
    char *target, *c;
    int matched;
    
    target = strdup((char *)xv_get(scan_ip->list_item,
	PANEL_LIST_STRING, line));
    c = target - 1;
    while ( *(++c) != '\0' )
	if ( isupper(*c) ) *c = tolower(*c);

    matched = (strstr(target, string) != CNULL);
    free(target);
    
    if (matched) {
	if (clear_msgs) select_all_msgs(FALSE);
	cur_msg = line;
	xv_set(scan_ip->list_item, PANEL_LIST_SELECT, cur_msg, TRUE, NULL);
	if (smsg_is_up()) show_smsg_window();
    }
    return matched;
}

/*
 * Notify callback function for `find_fwd_item'.
 */
void
find_fwd_notify(item, event)
Panel_item	item;
Event		*event;
{
    char *ss, *c;
    int temp_msg, failed;

    ss = strdup((char *)xv_get(find_ip->find_text_item, PANEL_VALUE));
    c = ss - 1;
    while ( *(++c) != '\0' )
	if ( isupper(*c) ) *c = tolower(*c);

    failed = FALSE;
    temp_msg = (cur_msg < num_msgs - 1 && cur_msg >= 0) ? cur_msg + 1 : 0;
    while (temp_msg < num_msgs) {
	if (scan_find(temp_msg, ss, TRUE)) break;
	temp_msg++;
    }

    if (temp_msg == num_msgs) {
	if (cur_msg > 0) {
	    temp_msg = 0;
	    while (temp_msg <= cur_msg) {
		if (scan_find(temp_msg, ss, TRUE)) break;
		temp_msg++;
	    }
	    if (temp_msg > cur_msg) failed = TRUE;
	}
	else
	    failed = TRUE;
    }
    if (failed)
	pr_msg(scan_ip->frame, TRUE, "No match.");
    else {
	pr_msg(scan_ip->frame, FALSE, "");
	num_selected = 1;
	set_scanwin_items_status();
    }
}

/*
 * User-defined action for `find_bkw_item'.
 */
void
find_bkw_notify(item, event)
Panel_item	item;
Event		*event;
{
    char *ss, *c;
    int temp_msg, failed;

    ss = strdup((char *)xv_get(find_ip->find_text_item, PANEL_VALUE));
    c = ss - 1;
    while ( ++*c != '\0' )
	if ( isupper(*c) ) tolower(*c);

    failed = FALSE;
    temp_msg = (cur_msg > 0 && cur_msg <= num_msgs - 1) ?
	cur_msg - 1 : num_msgs - 1;
    while (temp_msg >= 0) {
	if (scan_find(temp_msg, ss, TRUE))
	    break;
	temp_msg--;
    }

    if ( temp_msg == -1 ) {
	if (cur_msg > 0) {
	    temp_msg = num_msgs - 1;
	    while (temp_msg >= cur_msg) {
		if (scan_find(temp_msg, ss, TRUE))
		    break;
		temp_msg--;
	    }
	    if (temp_msg < cur_msg) failed = TRUE;
	}
	else
	    failed = TRUE;
    }
    if (failed)
	pr_msg(scan_ip->frame, TRUE, "No match.");
    else {
	pr_msg(scan_ip->frame, FALSE, "");
	num_selected = 1;
	set_scanwin_items_status();
    }
}

/*
 * Notify callback function for `find_all_item'.
 */
void
find_all_notify(item, event)
Panel_item      item;
Event		*event;
{
    char *ss, *c;
    int i, found;
    
    ss = strdup((char *)xv_get(find_ip->find_text_item, PANEL_VALUE));
    c = ss - 1;
    while ( ++*c != '\0' )
	if ( isupper(*c) ) tolower(*c);

    found = 0;
    for ( i = 0 ; i < num_msgs; i++)
	if (scan_find(i, ss, found == 0)) found++;

    if (found == 0)
	pr_msg(scan_ip->frame, TRUE, "No match.");
    else {
	pr_msg(scan_ip->frame, FALSE, "");
	num_selected = found;
	set_scanwin_items_status();
    }
}

/*
 * Notify callback function for `find_text_item'.
 */
Panel_setting
find_text_notify(item, event)
Panel_item	item;
Event		*event;
{
    if (event_shift_is_down(event))
        find_bkw_notify((Panel_item)NULL, (Event *)NULL);
    else
	find_fwd_notify((Panel_item)NULL, (Event *)NULL);

    if ((int)xv_get(find_ip->find_frame, FRAME_CMD_PIN_STATE) ==
	FRAME_CMD_PIN_OUT)
	xv_set(find_ip->find_frame, XV_SHOW, FALSE, NULL);
    return PANEL_NONE;
}
