/*
 *  msgboard_xv.c - XView version of the Message Board.
 */

#include <stdio.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <X11/Xlib.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/xv_xrect.h>
#include <pwd.h>
#include "msgboard_xv.h"
#include "msgboard.h"
#include "outs_menu.h"

/*
 * Instance XV_KEY_DATA key.  An instance is a set of related
 * user interface objects.  A pointer to an object's instance
 * is stored under this key in every object.  This must be a
 * global variable.
 */
Attr_attribute	INSTANCE;

Notify_func timer_notify();

struct itimerval timer;

Frame	base_frame = NULL;
Display *dpy;

int
    cur_sel = BOGUS,			/* Index of currently selected user */ 
    cur_cat = MB_ALL,			/* current category -- default ALL  */
    cur_status = MB_ALL,		/* current category -- default ALL  */
    delay = CK_INT,			/* Time between each database
					   check			*/
    fmt_msgs = TRUE,			/* Format phone messages?	*/
    pop_mod = TRUE,			/* Popup modify window when a list
					   entry is toggled to Out?	*/
    users = 0,				/* Number of users in datalist	*/
    numcats = 0,			/* Number of categories		*/
    watchnum = 0,			/* Number of logids on watchlist */
    admin = FALSE,			/* Does user have admin caps?	*/
    admin_enable = TRUE,		/* and do they want to use them? */
    prt_dest = TO_PRINTER,		/* print to printer or file?	*/
    prt_type = POSTSCRIPT,		/* print postscript or ascii?	*/
    wgrid,				/* spacing for panel items, in pixels */
    dragstyle = BOGUS;			/*  What DnD style should we use? */

char
    mb_server[MAXHOSTNAMELEN+1],	/* Name of the server host	*/
    mylogid[L_cuserid],			/* Current user's login ID	*/
    **cats,				/* Pointer to array of
					   category names		*/
    **watchlist,			/* Pointer to array of logids
					   to watch for			*/
    prt_name[PRINTERNAME_LEN+1],	/* Name of printer to print
					   msgs to			*/
    prt_dir[MAXPATHLEN],		/* Name of directory to print
    					   msgs to			*/
    prt_file[81];			/* Name of file to print
					   msgs to			*/

struct winparams	main_params, send_params, scan_params, page_params,
			outmod_params;

Mem_Rec
    *ulist,			/*  Pointer to list of users		*/
    *myentry;			/*  Pointer to current user's record in
				    ulist array				*/

Global_Rec
    globrec;			/*  Global data from the server		*/

/*
 *  Displays an error message in the footer of "frame".  If frame is NULL,
 *  use the main frame.  If we're out of the window system, just print.
 */
void
pr_msg(frame, bell, format, a1, a2, a3)
Frame frame;
int bell;
char *format, *a1, *a2, *a3;
{
    if (base_frame != NULL) {
	char msg[512];

	sprintf(msg, format, a1, a2, a3);
	if ( frame == NULL ) frame = base_frame;
	if (bell) window_bell(frame);
	xv_set(frame, FRAME_LEFT_FOOTER, msg, NULL);
    }
    else {
	if (bell) fputc('\007', stderr);
	fprintf(stderr, format, a1, a2, a3);
    }
}

/*
 *  Initializes/resets the delay between automatic checks of the database
 */
void
set_timer(flag)
int flag;
{
    if (flag) {
	timer.it_value.tv_sec = 1;
	timer.it_value.tv_usec = 0;
	timer.it_interval.tv_sec = delay;
	timer.it_interval.tv_usec = 0;
	(void)notify_set_itimer_func(base_frame, timer_notify, ITIMER_REAL,
	    &timer, ITIMER_NULL);
    }
    else
	(void)notify_set_itimer_func(base_frame, NOTIFY_FUNC_NULL,
	    ITIMER_REAL, NULL, ITIMER_NULL);
}

void
set_prt_defaults()
{
    char *ptr;

    ptr = getenv("PRINTER");
    if (ptr == CNULL)
	strcpy(prt_name, "lp");
    else {
	strncpy(prt_name, ptr, PRINTERNAME_LEN);
	prt_name[PRINTERNAME_LEN] = '\0';
    }
    getwd(prt_dir);
    strcpy(prt_file, "msgs.ps");
}

/*
 *  Save program parameters out to user parameter file
 */
void
save_params()
{
    FILE *file;
    char *ptr, filename[MAXPATHLEN + 1];
    struct winparams params;
    int i;

    ptr = getenv("HOME");
    if (ptr == CNULL) {
	pr_msg(NULL, TRUE, "Can't find your home directory!");
	return;
    }

    sprintf(filename, "%s/%s", ptr, PARAMFILE);
    if ( (file = fopen(filename, "w")) == FNULL) {
	pr_msg(NULL, TRUE, "Could not open %s", filename);
	return;
    }

    fprintf(file, "delay: %d\n", delay);
    fprintf(file, "pop_mod: %d\n", pop_mod);
    fprintf(file, "fmt_msgs: %d\n", fmt_msgs);
    fprintf(file, "admin_enable: %d\n", admin_enable);
    get_main_params(&params);
    fprintf(file, "mainwin: %d %d %d %d %d\n", params.state,
	params.rect.r_left, params.rect.r_top, params.rect.r_width,
	params.rect.r_height);
    get_send_params(&params);
    fprintf(file, "sendwin: %d %d %d %d %d\n", params.state,
	params.rect.r_left, params.rect.r_top, params.rect.r_width,
	params.rect.r_height);
    get_scan_params(&params);
    fprintf(file, "scanwin: %d %d %d %d %d\n", params.state,
	params.rect.r_left, params.rect.r_top, params.rect.r_width,
	params.rect.r_height);
    get_page_params(&params);
    fprintf(file, "pagewin: %d %d %d %d %d\n", params.state,
	params.rect.r_left, params.rect.r_top, params.rect.r_width,
	params.rect.r_height);
    get_outmod_params(&params);
    fprintf(file, "outmodwin: %d %d %d %d %d\n", params.state,
	params.rect.r_left, params.rect.r_top, params.rect.r_width,
	params.rect.r_height);

    for ( i = 0 ; i < num_outs ; i++ )
        fprintf(file, "outs: %1s|%1s|%1s|%d\n",
	    outlist[i].label, outlist[i].until,
            outlist[i].message, outlist[i].pop);

    fclose(file);
}

/*
 *  Read program parameters in from PARAMFILE
 */
void
load_params()
{
    FILE *file;
    char *ptr, keywd[21], filename[MAXPATHLEN+1], buf[151];

    ptr = getenv("HOME");
    if (ptr == CNULL)
	return;

    send_params.state = BOGUS;
    scan_params.state = BOGUS;
    page_params.state = BOGUS;
    outmod_params.state = BOGUS;
    main_params.state = BOGUS;

    sprintf(filename, "%s/%s", ptr, PARAMFILE);
    if ( (file = fopen(filename, "r")) == FNULL)
	return;

    while (fgets(buf, 150, file) != CNULL) {
	if (sscanf(buf, "%20[^:]", keywd) == 1) {
	    if (strcmp("delay", keywd) == 0) {
		sscanf(buf, "%*s %d", &delay);
	    }
	    else if (strcmp("pop_mod", keywd) == 0) {
		sscanf(buf, "%*s %d", &pop_mod);
	    }
	    else if (strcmp("fmt_msgs", keywd) == 0) {
		sscanf(buf, "%*s %d", &fmt_msgs);
	    }
	    else if (strcmp("admin_enable", keywd) == 0) {
		sscanf(buf, "%*s %d", &admin_enable);
	    }
	    else if (strcmp("outs", keywd) == 0)
		read_outs_line(buf);
	    else if (strcmp("mainwin", keywd) == 0) {
		sscanf(buf, "%*s %d %hd %hd %hd %hd", &(main_params.state),
		    &(main_params.rect.r_left), &(main_params.rect.r_top),
		    &(main_params.rect.r_width),
		    &(main_params.rect.r_height));
	    }
	    else if (strcmp("sendwin", keywd) == 0) {
		sscanf(buf, "%*s %d %hd %hd %hd %hd", &(send_params.state),
		    &(send_params.rect.r_left), &(send_params.rect.r_top),
		    &(send_params.rect.r_width),
		    &(send_params.rect.r_height));
	    }
	    else if (strcmp("scanwin", keywd) == 0) {
		sscanf(buf, "%*s %d %hd %hd %hd %hd", &(scan_params.state),
		    &(scan_params.rect.r_left), &(scan_params.rect.r_top),
		    &(scan_params.rect.r_width),
		    &(scan_params.rect.r_height));
	    }
	    else if (strcmp("pagewin", keywd) == 0) {
		sscanf(buf, "%*s %d %hd %hd %hd %hd", &(page_params.state),
		    &(page_params.rect.r_left), &(page_params.rect.r_top),
		    &(page_params.rect.r_width),
		    &(page_params.rect.r_height));
	    }
	    else if (strcmp("outmodwin", keywd) == 0) {
		sscanf(buf, "%*s %d %hd %hd %hd %hd", &(outmod_params.state),
		    &(outmod_params.rect.r_left), &(outmod_params.rect.r_top),
		    &(outmod_params.rect.r_width),
		    &(outmod_params.rect.r_height));
	    }
	}
    }
    fclose(file);
}

void
kaboom()
{
    if (base_frame != NULL) {
	xv_destroy_safe(base_frame);
	fprintf(stderr, "Program exited prematurely\n");
	sleep(3);
    }
   exit(1);
}

/****************
 *  Main
 ****************/
void
main(argc, argv)
int	argc;
char	**argv;
{
    int release;

    if (cuserid(mylogid) == CNULL) {
	pr_msg(NULL, TRUE, "System doesn't know who %s is! (cuserid failed)",
	    mylogid);
	kaboom();
    }

    /*
     *  Initialize XView.
     */
    xv_init(XV_INIT_ARGC_PTR_ARGV, &argc, argv, 0);
    INSTANCE = xv_unique_key();

    /*
     *  Read the user's parameter file.
     */
    load_params();

    /*
     *  Initialize user interface components.
     */
    create_main_window();
    dpy = (Display *)xv_get(base_frame, XV_DISPLAY);

    /*
     *  Determine the default drag and drop style.  Use the V2 style only
     *  if we're running on a V2 server.
     */
    dragstyle = VendorRelease(dpy);
    if (dragstyle >= 2000 && dragstyle < 3000)
	dragstyle = 2;
    else
	dragstyle = 3;

    /*
     *  Disable reading of any mail initialization files
     */
    putenv("MAILRC=/dev/null");

    /*
     *  More initialization...
     */
    create_signs();
    set_prt_defaults();

    /*
     *  Try connecting to the specified server; ask it for info.
     *  "Modified records," since we're just firing up, means all
     *  records.
     */
    if (connect_to_server()) (void)get_modified_records();

    /*
     *  Set the active/inactive status of main panel items to jibe with
     *  whether or not any item on the scrolling is selected.
     */
    set_mainwin_items_status();

    /*
     *  Turn on the update timer.
     */
    set_timer(TRUE);

    /*
     *  Should we bring up any of the popup windows?
     */
    if (send_params.state == FRAME_CMD_PIN_IN)
	show_sendmsg_window();
    if (scan_params.state == FRAME_CMD_PIN_IN)
	show_scan_window();
    if (page_params.state == FRAME_CMD_PIN_IN)
	show_page_window();
    if (outmod_params.state == FRAME_CMD_PIN_IN)
	show_outmod_window((Mem_Rec *)NULL);
    
    /*
     *  Hand control over to XView.
     */
    xv_main_loop(base_frame);
    base_frame = NULL;
    exit(0);
}

/*
 *  This should be called whenever a user event is detected, to clear
 *  warning-level messages.
 */
void
clear_msg()
{
    xv_set(base_frame, FRAME_LEFT_FOOTER, "", NULL);
}

void
look_busy(flag)
int flag;
{
   xv_set(base_frame, FRAME_BUSY, flag, NULL);
}

/*
 *  See if there are any changes to the database.  Try to connect to server
 *  if we're disconnected.
 */
Notify_func
timer_notify()
{
    if (get_modified_records() == MSGD_DISCONNECTED) {
	look_busy(TRUE);
	pr_msg(NULL, FALSE, "Attempting to connect to %s...", mb_server);
	if (connect_to_server()) {
	    pr_msg(NULL, FALSE, "Connected to %s.", mb_server);
	    (void)get_modified_records();
	}
	look_busy(FALSE);
    }

    return NOTIFY_DONE;
}

/*
 *  Positions Apply/Reset type buttons.  Uses: wgrid
 */
void
position_bot_buttons(panel, b1, b2, r)
Panel panel;
Panel_item b1, b2;
int r;
{
    int b1w, b1x;

    b1w = (int)xv_get(b1, XV_WIDTH);
    b1x = ((int)xv_get(panel, XV_WIDTH) - b1w - 
	(int)xv_get(b2, XV_WIDTH) - wgrid) / 2;
    xv_set(b1,
	XV_X, b1x,
	NULL);
    xv_set(b2,
	XV_X, b1x + b1w + wgrid,
	NULL);
}
	
#ifdef SCALABLE
/*
 *  Align items by their values.  Pick your item with the longest label,
 *  position it, and pass its PANEL_VALUE_X as "valx" for all your other
 *  items.
 */
void
align_values(item, valx, ypos)
Panel_item item;
int valx, ypos;
{
    xv_set(item,
	XV_X, (int)xv_get(item, XV_X) + valx - (int)xv_get(item, PANEL_VALUE_X),
	XV_Y, ypos,
	NULL);
}
#endif

/*
 *  Deletes a login ID from the watch list.
 */
int
watch_del(u)
char *u;
{
    char **rp;
    int i;

    rp = (char **)lfind((char *)&u, (char *)watchlist, &watchnum,
	sizeof(char *),	comp_strings);

    if (rp == (char **)NULL) return FALSE;

    free(*rp);
    watchnum--;
    for ( i = (rp - watchlist); i < watchnum; i++)
	watchlist[i] = watchlist[i+1];

    if ( watchnum == 0 && (int)xv_get(base_frame, FRAME_CLOSED) )
	set_timer(FALSE);

    return TRUE;
}

/*
 *  The outs button handler calls this to commit the appropriate values
 *  to the database.
 */
void
set_from_out_button(until, comment, pop)
char *until, *comment;
int pop;
{
    Mem_Rec *rec;

    if (admin) {
	if ( cur_sel != BOGUS )
	    rec = &ulist[cur_sel];
	else {
	    pr_msg(NULL, TRUE, "No one selected on list.");
	    return;
	}
    }
    else {
	if (myentry == (Mem_Rec *)NULL) {
	    pr_msg(NULL, TRUE, "%s is not currently on the Message board.",
		mylogid);
	    return;
	}
	rec = myentry;
    }

    modify_status(rec->logid, MB_OUT, until, comment);
    
    if (get_modified_records() == 0 && pop) show_outmod_window(rec);
}

/*
 *  Called by client library routines whenever server connection is lost.
 */
void
server_was_dropped()
{
    if (users > 0) {
	free(ulist);
	users = 0;
	myentry = NULL;
    }
    globrec.name[0] = '\0';
    globrec.motd[0] = '\0';
    update_screen();
    update_global_display();
}
