/*
 *  pagewin.c - Pager-dialing window for the Message Board.
 */

#include <stdio.h>
#include <sys/param.h>
#include <sys/types.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include "pagewin_ui.h"
#include "msgboard.h"
#include "msgboard_xv.h"

/*
 * Compatibility defines to match msgboard code to beeptool's:
 */
#define SelectedType	(pagee.pagertype)
#define SelectedLogin	(pagee.logid)
#define SelectedName	(pagee.fname)

#define MAX_PAGE_STRING	150

char *PagerTypes[] = {
	"alpha",
	"numeric", 
	"skypager"
};

pagewin_frame_objects		*page_ip;

/*
 * Info about the person to page (the "pagee"):
 */
Mem_Rec pagee;

int
    page_window_created = FALSE;	/* is page window created?	*/

void
create_page_window()
{
#ifdef SCALABLE
    int valx, ypos;
#endif

    page_ip = pagewin_frame_objects_initialize(NULL, base_frame);

    if (page_params.state != BOGUS) {
	frame_set_rect(page_ip->frame, &page_params.rect);
	if (page_params.state == TRUE)
	    xv_set(page_ip->frame, FRAME_CMD_PIN_STATE, FRAME_CMD_PIN_IN, NULL);
    }

#ifdef SCALABLE
    xv_set(page_ip->textfield1, XV_X, wgrid * 2, NULL);
    valx = (int)xv_get(page_ip->textfield1, PANEL_VALUE_X);

    ypos = wgrid * 2;
    align_values(page_ip->to_item, valx, ypos);
    xv_set(page_ip->fname_item,
	XV_X,	(int)xv_get(page_ip->to_item, XV_X) +
		(int)xv_get(page_ip->to_item, XV_WIDTH) + wgrid,
	XV_Y,	wgrid * 2,
	NULL);
    ypos += (int)xv_get(page_ip->to_item, XV_HEIGHT) + wgrid;
    align_values(page_ip->numeric, valx, ypos);
    align_values(page_ip->textfield1, valx, ypos);
    ypos += (int)xv_get(page_ip->numeric, XV_HEIGHT) + wgrid;
    align_values(page_ip->textfield2, valx, ypos);
    ypos += (int)xv_get(page_ip->textfield2, XV_HEIGHT) + wgrid * 2;
    xv_set(page_ip->send_item, XV_Y, ypos, NULL);
    xv_set(page_ip->cancel_item, XV_Y, ypos, NULL);
    window_fit(page_ip->panel);
    window_fit(page_ip->frame);
    position_bot_buttons(page_ip->panel, page_ip->send_item,
	page_ip->cancel_item);
#endif

    page_window_created = TRUE;
}

/*
 *  Gets the necessary info for the person we want to page.
 */
void
set_pagewin_items()
{
    char cbuf[FNAME_LEN+3], typebuf[15];
    Mem_Rec *mp;

    strncpy(pagee.logid, (char *)xv_get(page_ip->to_item, PANEL_VALUE),
	LOGID_LEN);
    pagee.logid[LOGID_LEN] = '\0';
    mp = find_urec(pagee.logid);
    if (mp == (Mem_Rec *)NULL) {
        xv_set(page_ip->fname_item, PANEL_LABEL_STRING, "(???)", NULL);
	pagee.pagernum[0] = '\0';
    }	
    else {
	pagee = *mp;
	sprintf(cbuf, "(%s)", pagee.fname);
	xv_set(page_ip->fname_item, PANEL_LABEL_STRING, cbuf, NULL);
    }

    if (pagee.pagernum[0] == '\0') {
	pr_msg(page_ip->frame, TRUE, "No pager number found for %s.",
	    pagee.logid);
 	xv_set(page_ip->frame, FRAME_RIGHT_FOOTER, "", NULL);
	xv_set(page_ip->send_item, PANEL_INACTIVE, TRUE, NULL);
	xv_set(page_ip->textfield1, XV_SHOW, FALSE, NULL);
	xv_set(page_ip->textfield2, XV_SHOW, FALSE, NULL);
	xv_set(page_ip->numeric,
	    PANEL_VALUE,	"",
	    PANEL_INACTIVE,	TRUE,
	    XV_SHOW, 		TRUE,
	    NULL);
    }
    else {
	sprintf(typebuf, "Type: %s", PagerTypes[pagee.pagertype]);
	xv_set(page_ip->frame, FRAME_RIGHT_FOOTER, typebuf, NULL);
	pr_msg(page_ip->frame, FALSE, "");

	xv_set(page_ip->send_item, PANEL_INACTIVE, FALSE, NULL);

	if (pagee.pagertype != BEEP_ALPHA) {
	    xv_set(page_ip->textfield1,
		XV_SHOW,	FALSE,
		NULL);
	    xv_set(page_ip->textfield2,
		XV_SHOW,	FALSE,
		NULL);
	    xv_set(page_ip->numeric,
		PANEL_VALUE,	"",
		PANEL_INACTIVE,	FALSE,
		XV_SHOW ,	TRUE,
		NULL);
	    xv_set(page_ip->panel,
	    PANEL_CARET_ITEM, page_ip->numeric,
	    NULL);
	}
	else {
	    xv_set(page_ip->numeric,
		XV_SHOW ,	FALSE,
		NULL);
	    xv_set(page_ip->textfield1,
		PANEL_VALUE,	"",
		XV_SHOW,	TRUE,
		NULL);
	    xv_set(page_ip->textfield2,
		PANEL_VALUE,	"",
		XV_SHOW,	TRUE,
		NULL);
	    xv_set(page_ip->panel,
		PANEL_CARET_ITEM, page_ip->textfield1,
		NULL);
	}
    }
}

void
show_page_window()
{
    if (!page_window_created) create_page_window();

    if (cur_sel != BOGUS)
	xv_set(page_ip->to_item, PANEL_VALUE, ulist[cur_sel].logid, NULL);
    set_pagewin_items();
    xv_set(page_ip->frame, XV_SHOW, TRUE, NULL);
}

/*
 * Notify callback function for `send_button'.
 */
void
send_message_handler(item, event)
Panel_item	item;
Event		*event;
{
    char		message[MAX_PAGE_STRING];
    char		*ptr;
	
    switch (SelectedType) {
    case BEEP_ALPHA:
	sprintf(message, "%s %s",
	    (char *)xv_get(page_ip->textfield1, PANEL_VALUE),
	    (char *)xv_get(page_ip->textfield2, PANEL_VALUE));
	break;

    case BEEP_NUM:
    case BEEP_SKY:
	ptr = (char *)xv_get(page_ip->numeric, PANEL_VALUE);
	if (ptr[0] == NULL)
	    strcpy(message, MESSAGE);
	else
	    strcpy(message, ptr);

	break;

    default:
	pr_msg(page_ip->frame, TRUE, "Unknown pager type defined for %s.",
	    SelectedName);
	return;
	break;
    }

    if ( sendpage(SelectedLogin, message) == 0 )
	pr_msg(page_ip->frame, FALSE, "page sent to %s", SelectedName);
}

/*
 * Notify callback function for `to_item'.
 */
Panel_setting
pagewin_to_notify(item, event)
Panel_item	item;
Event		*event;
{
    set_pagewin_items();
    if (event_shift_is_down(event))
	return PANEL_PREVIOUS;
    else
	return PANEL_NEXT;
}

/*
 *  Gathers all the parameters for this window
 */
void
get_page_params(p)
struct winparams *p;
{
    if (!page_window_created)
	*p = page_params;
    else {
	p->state = (int)xv_get(page_ip->frame, FRAME_CMD_PIN_STATE);
	frame_get_rect(page_ip->frame, &(p->rect));
    }
}
