/*
 * pmsg_prt.c - Phone message printing and drag'n'drop preparation routines.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/param.h>
#include <sys/types.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include "scanwin_ui.h"
#include "pmsg_prolog.h"
#include "msgboard.h"
#include "msgboard_xv.h"
#include "pmsg_client.h"

/*
 *  Pointer to scan window objects:
 */
extern scanwin_frame_objects		*scan_ip;
extern scanwin_copy_frame_objects	*copy_ip;

extern int num_msgs, msg_cat;
extern char cur_msg_user[];

/*
 *  These should be the same as the ones declared in pmsg_prt.c !
 */
char *messages[] = {
	"No message",
	"Returned your call",
	"Please call back",
	"Will call again",
	"Left following message"
	};

/*
 * Make sure parentheses and backslashes are escaped in strings heading for
 * PostScript-land.  The passed string MUST be one that has been malloc'ed!
 */
void
prep_string_4_ps(s)
char *s;
{
    char *tmp;
    int i, j, l;

    if (strpbrk(s, "()\\") != CNULL) {
	tmp = malloc(strlen(s) * 2);
	j = -1;
	l = strlen(s);
	for (i = 0; i < l; i++) {
	    if (s[i] == '(' || s[i] == ')' || s[i] == '\\')
	    tmp[++j] = '\\';
	    tmp[++j] = s[i];
	}
	tmp[++j] = '\0';
	free(s);
	s = realloc(tmp, strlen(tmp) + 1);
    }
}

/*
 * Print selected messages, in PostScript or pretty crude ASCII format.
 */
void
print_messages()
{
    int msg_to_print, msgs_printed, i;
    char *p, fullname[FNAME_LEN+1];
    FILE *outfile;
    struct pmsg pmsg;

    msg_to_print = (int)xv_get(scan_ip->list_item, PANEL_LIST_FIRST_SELECTED);
    if (msg_to_print == -1) {
	pr_msg(scan_ip->frame, TRUE, "No messages selected for printing");
	return;
    }

    if (prt_dest == TO_PRINTER) {
	p = malloc(strlen(prt_name) + 7);
	sprintf(p, "lpr -P%s", prt_name);
	outfile = popen(p, "w");
    }
    else {
	if (prt_file[0] == '/') {
	    p = malloc(strlen(prt_file) + 1);
	    strcpy(p, prt_file);
	}
	else {
	    p = malloc(strlen(prt_dir) + strlen(prt_file) + 2);
	    sprintf(p, "%s/%s", prt_dir, prt_file);
	}
	outfile = fopen(p, "w");
    }
    free(p);

    if (outfile == FNULL) {
	if (prt_dest == TO_PRINTER)
	    pr_msg(scan_ip->frame, TRUE, "Could not open pipe to printer!");
	else
	    pr_msg(scan_ip->frame, TRUE, "Could not open print output file!");
	return;
    }

    if (prt_type == POSTSCRIPT)
	fputs(pmsg_prolog, outfile);

    strcpy(fullname, get_fname(cur_msg_user));

    msgs_printed = 0;
    while (msg_to_print != -1) {
	get_pmsg(cur_msg_user, msg_cat,
	    (char *)xv_get(scan_ip->list_item,
		PANEL_LIST_CLIENT_DATA, msg_to_print),
	    &pmsg);

	if (prt_type == POSTSCRIPT) {
	    fprintf(outfile, "StartMsg\n");
	    prep_string_4_ps(pmsg.id);
	    fprintf(outfile, "(%s) PrintSig\n", pmsg.id);
	    prep_string_4_ps(fullname);
	    fprintf(outfile, "(%s) PrintFor\n", fullname);
	    prep_string_4_ps(pmsg.caller);
	    fprintf(outfile, "(%s) PrintCaller\n", pmsg.caller);
	    prep_string_4_ps(pmsg.time);
	    fprintf(outfile, "(%s) PrintTime\n", strtok(pmsg.time, " "));
	    fprintf(outfile, "(%s) PrintDate\n", strtok(CNULL, "\n"));
	    prep_string_4_ps(pmsg.org);
	    fprintf(outfile, "(%s) PrintOrg\n", pmsg.org);
	    prep_string_4_ps(pmsg.phnum);
	    fprintf(outfile, "(%s) PrintPhone\n", pmsg.phnum);
	    for (i = 0; i <= SM_MAX_CHOICE; i++)
		if ( (pmsg.flags & (2 << i)) != 0 )
		    fprintf(outfile, "%d MarkMsg\n", i);
	    if ((pmsg.flags & SM_CHOICE_LV_MSG) != 0) {
		fprintf(outfile, "(%s) PrintMsg\n", pmsg.text);
	    }
	    fprintf(outfile, "FinishMsg\n\n");
	}
	else {
	    fprintf(outfile, "  Message by: %s\n", pmsg.id);
	    fprintf(outfile, "          To: %s\n", fullname);
	    fprintf(outfile, "      Caller: %s\n", pmsg.caller);
	    fprintf(outfile, "        Time: %s\n", pmsg.time);
	    fprintf(outfile, "Organization: %s\n", pmsg.org);
	    fprintf(outfile, "       Phone: %s\n", pmsg.phnum);
	    for (i = 0; i <= SM_MAX_CHOICE; i++)
		if ( (pmsg.flags & (2 << i)) != 0 )
		    fprintf(outfile, "     Message: %s\n", messages[i]);
	    if ((pmsg.flags & SM_CHOICE_LV_MSG) != 0)
		fprintf(outfile, "\n%s\n\n", pmsg.text);
	}

	free_pmsg(&pmsg);
	msgs_printed++;
	msg_to_print = (int)xv_get(scan_ip->list_item,
	    PANEL_LIST_NEXT_SELECTED, msg_to_print);
    }

    if (prt_type == POSTSCRIPT)
	fprintf(outfile, "EndItAll\n");

    if (prt_dest == TO_PRINTER)
	pclose(outfile);
    else
	fclose(outfile);

    if (msgs_printed == 1)
	pr_msg(scan_ip->frame, FALSE, "One message printed");
    else
	pr_msg(scan_ip->frame, FALSE, "%d messages printed", msgs_printed);
}

/*
 *  Prepare a file to drag'n'drop (V2).
 */
void
dnd_messages(outname)
char *outname;
{
    int msg_to_print, msgs_printed;
    FILE *outfile;
    struct pmsg pmsg;

    msg_to_print = (int)xv_get(scan_ip->list_item, PANEL_LIST_FIRST_SELECTED);
    if (msg_to_print == -1) {
	pr_msg(scan_ip->frame, TRUE, "No messages selected for dragging");
	return;
    }

    outfile = fopen(outname, "w");
    if (outfile == FNULL) {
	pr_msg(scan_ip->frame, TRUE, "Could not open output file!");
	return;
    }

    msgs_printed = 0;
    while (msg_to_print != -1) {
	get_pmsg(cur_msg_user, msg_cat,
	    (char *)xv_get(scan_ip->list_item,
		PANEL_LIST_CLIENT_DATA, msg_to_print),
	    &pmsg);

	fprintf(outfile, "%s\n", pmsg.caller);
	fprintf(outfile, "%s\n", pmsg.time);
	fprintf(outfile, "%s\n", pmsg.org);
	fprintf(outfile, "%s\n", pmsg.phnum);
	if ((pmsg.flags & SM_CHOICE_LV_MSG) != 0)
	    fprintf(outfile, "\n%s\n", pmsg.text);

	fprintf(outfile, "\n\n");

	free_pmsg(&pmsg);
	msgs_printed++;
	msg_to_print = (int)xv_get(scan_ip->list_item,
	    PANEL_LIST_NEXT_SELECTED, msg_to_print);
    }
    fclose(outfile);

    if (msgs_printed == 1)
	pr_msg(scan_ip->frame, FALSE, "One message copied");
    else
	pr_msg(scan_ip->frame, FALSE, "%d messages copied", msgs_printed);
}

/*
 *  Prepare a buffer containing the currently selected messages.  Returns
 *  pointer to malloced string, and assigns length of the string to
 *  location "length" points to.
 */
char *
copy_messages(length)
unsigned long *length;
{
    int msg_to_print, msgs_printed, buflen, obuflen;
    struct pmsg pmsg;
    char *ptr;

    msg_to_print = (int)xv_get(scan_ip->list_item, PANEL_LIST_FIRST_SELECTED);
    if (msg_to_print == -1) {
	pr_msg(scan_ip->frame, TRUE, "No messages selected for clipboard");
	return;
    }

    ptr = strdup("");
    buflen = 1;

    msgs_printed = 0;
    while (msg_to_print != -1) {
	get_pmsg(cur_msg_user, msg_cat,
	    (char *)xv_get(scan_ip->list_item,
		PANEL_LIST_CLIENT_DATA, msg_to_print),
	    &pmsg);

	obuflen = buflen;
	buflen += strlen(pmsg.caller) + strlen(pmsg.time)
	    + strlen(pmsg.org) + strlen(pmsg.phnum) + 6;
	ptr = realloc(ptr, buflen);
	sprintf(ptr + obuflen - 1, "%s\n%s\n%s\n%s\n",
	    pmsg.caller, pmsg.time, pmsg.org, pmsg.phnum);

	if ((pmsg.flags & SM_CHOICE_LV_MSG) != 0) {
	    obuflen = buflen;
	    buflen += strlen(pmsg.text) + 2;
	    ptr = realloc(ptr, buflen);
	    sprintf(ptr + obuflen - 3, "\n%s\n", pmsg.text);
	}

	sprintf(ptr + buflen - 3, "\014\n");

	free_pmsg(&pmsg);
	msgs_printed++;
	msg_to_print = (int)xv_get(scan_ip->list_item,
	    PANEL_LIST_NEXT_SELECTED, msg_to_print);
    }

    if (msgs_printed == 1)
	pr_msg(scan_ip->frame, FALSE, "One message copied");
    else
	pr_msg(scan_ip->frame, FALSE, "%d messages copied",
	    msgs_printed);

    *length = buflen - 1;
    return ptr;
}

/*
 *  Copy selected messages to specified file.
 */
void
sc_copy_to_file()
{
    int msg_to_print, msgs_printed, i;
    char fullname[FNAME_LEN+1], outfilename[MAXPATHLEN];
    FILE *outfile;
    struct pmsg pmsg;

    msg_to_print = (int)xv_get(scan_ip->list_item, PANEL_LIST_FIRST_SELECTED);
    if (msg_to_print == -1) {
	pr_msg(scan_ip->frame, TRUE, "No messages selected for printing");
	return;
    }

    strcpy(outfilename, (char *)xv_get(copy_ip->copy_file_item, PANEL_VALUE));
    if (outfilename[0] != '/')
	sprintf(outfilename, "%s/%s",
	    (char *)xv_get(copy_ip->copy_dir_item, PANEL_VALUE),
	    (char *)xv_get(copy_ip->copy_file_item, PANEL_VALUE));
    outfile = fopen(outfilename, "w");

    if (outfile == FNULL) {
	pr_msg(scan_ip->frame, TRUE, "Could not open %s!", outfilename);
	return;
    }

    strcpy(fullname, get_fname(cur_msg_user));

    msgs_printed = 0;
    while (msg_to_print != -1) {
	get_pmsg(cur_msg_user, msg_cat,
	    (char *)xv_get(scan_ip->list_item,
		PANEL_LIST_CLIENT_DATA, msg_to_print),
	    &pmsg);

	    fprintf(outfile, "  Message by: %s\n", pmsg.id);
	    fprintf(outfile, "          To: %s\n", fullname);
	    fprintf(outfile, "      Caller: %s\n", pmsg.caller);
	    fprintf(outfile, "        Time: %s\n", pmsg.time);
	    fprintf(outfile, "Organization: %s\n", pmsg.org);
	    fprintf(outfile, "       Phone: %s\n", pmsg.phnum);
	    for (i = 0; i <= SM_MAX_CHOICE; i++)
		if ( (pmsg.flags & (2 << i)) != 0 )
		    fprintf(outfile, "     Message: %s\n", messages[i]);
	    if ((pmsg.flags & SM_CHOICE_LV_MSG) != 0)
		fprintf(outfile, "\n%s\n\n", pmsg.text);

	free_pmsg(&pmsg);
	msgs_printed++;
	msg_to_print = (int)xv_get(scan_ip->list_item,
	    PANEL_LIST_NEXT_SELECTED, msg_to_print);
    }

    fclose(outfile);

    if (msgs_printed == 1)
	pr_msg(scan_ip->frame, FALSE, "One message copied");
    else
	pr_msg(scan_ip->frame, FALSE, "%d messages copied", msgs_printed);
}
