/*
 *  scanwin.c --  support for the msgboard "scan message" window.
 */

#include <stdio.h>
#include <string.h>
#include <sys/param.h>
#include <sys/types.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/xv_xrect.h>
#include <xview/scrollbar.h>
#include <xview/cursor.h>
#include <xview/sel_pkg.h>
#include <xview/dragdrop.h>
#include <X11/Xlib.h>
#include "scanwin_ui.h"
#include "msgboard.h"
#include "msgboard_xv.h"
#include "pmsg_client.h"

Atom	targ_atom = NULL, len_atom, dragdone_atom;

#define TARGETS			targ_atom
#define LENGTH			len_atom
#define _SUN_DRAGDROP_DONE	dragdone_atom

short rolo_bits[] = {
#include "rolo_cursor.icon"
};

short mrolo_bits[] = {
#include "mrolo_cursor.icon"
};

short r_accept_bits[] = {
#include "r_accept_cursor.icon"
};

static char
	*scan_categories[] = {"new", "pending", "complete", "trash"};

Scrollbar	scan_list_bar;
Server_image	rolo_image, mrolo_image, r_accept_image;
Xv_Cursor	rolo_cursor, mrolo_cursor, r_accept_cursor;

scanwin_frame_objects		*scan_ip;
scanwin_find_frame_objects	*find_ip;
scanwin_copy_frame_objects	*copy_ip;

Menu_item
	/*
	 *  Items added to the scrolling list's menu:
	 */
	scsl_select_item = (Menu_item)NULL,
	scsl_show_item = (Menu_item)NULL,
	scsl_move_item = (Menu_item)NULL,
	scsl_print_item = (Menu_item)NULL,
	/*
	 *  Items on the move menu:
	 */
	scmm_next_item = (Menu_item)NULL,
	scmm_new_item = (Menu_item)NULL,
	scmm_pending_item = (Menu_item)NULL,
	scmm_complete_item = (Menu_item)NULL,
	scmm_trash_item = (Menu_item)NULL,
	scmm_clip_item = (Menu_item)NULL,
	scmm_file_item = (Menu_item)NULL;

char
    cur_msg_user[LOGID_LEN+1],		/*  Name of the person whose msgs
					    are currently displayed	*/
    *clip_buf = CNULL;			/*  Pointer to buffer for CLIPBOARD
					    transfers			*/

int
    scan_window_created = FALSE,	/* is scan window created?	*/
    find_window_created = FALSE,	/* is find window created?	*/
    copy_window_created = FALSE,	/* is copy window created?	*/
    scan_list_menu_updated = FALSE,	/* has menu for the scroll_list been
					   customized yet?		    */
    num_msgs,				/* How many items in list? */
    num_selected,			/* How many items are selected? */
    cur_msg = BOGUS,			/* Current message */
    not_my_msgs,			/* TRUE if we're not showing the */
					/* user's own messages		 */
    msg_cat,				/* current message category	*/
    clip_len;				/* Length of clip_buf string	*/

/*
 *  Sets the fullname string for the scanmsg popup
 */
void
set_scm_fname_item()
{
    char cbuf[FNAME_LEN+3];

    sprintf(cbuf, "(%s)",
	get_fname((char *)xv_get(scan_ip->user_item, PANEL_VALUE)));
    xv_set(scan_ip->fname_item, PANEL_LABEL_STRING, cbuf, NULL);
}

/*
 *  Updates the status of various objects on the window
 */
void
set_scanwin_items_status()
{
    char msgs[15], buf[32];
    int no_sel;

    if (!scan_window_created) return;

    if (num_msgs == 0)
	sprintf(msgs, "No messages");
    else if (num_msgs == 1)
	sprintf(msgs, "One message", num_msgs);
    else
	sprintf(msgs, "%d messages", num_msgs);
    if (num_selected > 0)
	sprintf(buf, "%s (%d selected)", msgs, num_selected);
    else
	strcpy(buf, msgs);

    xv_set(scan_ip->frame, FRAME_RIGHT_FOOTER, buf, NULL);

    no_sel = (num_selected == 0);

    xv_set(scan_ip->show_item, PANEL_INACTIVE, no_sel, NULL);
    xv_set(scan_ip->move_item, PANEL_INACTIVE, no_sel, NULL);
    xv_set(scan_ip->print_item, PANEL_INACTIVE, no_sel, NULL);

    xv_set(scan_ip->find_item,
	PANEL_INACTIVE,	num_msgs == 0,
	NULL);

    xv_set(scan_ip->torolo_item,
	PANEL_INACTIVE, no_sel,
	XV_SHOW,	dragstyle == 2,
	NULL);

    xv_set(scan_ip->drop_target1,
	PANEL_DROP_FULL,	!no_sel,
	XV_SHOW,		dragstyle == 3,
	NULL);

    xv_set(scan_ip->select_all_item,
	PANEL_INACTIVE,	num_msgs == num_selected,
	NULL);
    xv_set(scan_ip->clear_all_item,
	PANEL_INACTIVE,	num_selected == 0,
	NULL);

    if (scsl_select_item != (Menu_item)NULL)
	xv_set(scsl_select_item, MENU_INACTIVE, num_msgs == num_selected, NULL);
    if (scsl_show_item != (Menu_item)NULL)
	xv_set(scsl_show_item, MENU_INACTIVE, no_sel, NULL);
    if (scsl_move_item != (Menu_item)NULL)
	xv_set(scsl_move_item, MENU_INACTIVE, no_sel, NULL);
    if (scsl_print_item != (Menu_item)NULL)
	xv_set(scsl_print_item, MENU_INACTIVE, no_sel, NULL);

}

/*
 *  Chooses or clears all items on the list.
 */
void
select_all_msgs(flag)
int flag;
{
    int i;

    xv_set(scan_ip->list_item, XV_SHOW, FALSE, NULL);
    num_selected = 0;
    for ( i = 0; i < num_msgs; i++) {
	xv_set(scan_ip->list_item, PANEL_LIST_SELECT, i, flag, NULL);
	if (flag) num_selected++;
    }
    set_scanwin_items_status();
    xv_set(scan_ip->list_item, XV_SHOW, TRUE, NULL);
}

/*
 * Displays message selected on the list.
 */
void
display_message()
{
    int ret;

    cur_msg = (int)xv_get(scan_ip->list_item,
	PANEL_LIST_NEXT_SELECTED, cur_msg);
    if (cur_msg == -1)
	cur_msg = (int)xv_get(scan_ip->list_item, PANEL_LIST_FIRST_SELECTED);
    if (cur_msg == -1) {
	cur_msg = BOGUS;
	return;
    }

    ret = show_smsg_window();
    if ( ret != MSGD_OK)
	pr_msg(scan_ip->frame, TRUE, "Error %d retrieving message", ret);
    else
	pr_msg(scan_ip->frame, FALSE, "Showing message %d", cur_msg + 1);
}

/*
 *  Gets a summary line of each message.
 */
void
scm_get_messages()
{
    int new_size, i;
    struct pmsg_hdr *ph_ptr;

    strcpy(cur_msg_user, (char *)xv_get(scan_ip->user_item, PANEL_VALUE));
    if (strlen(cur_msg_user) == 0) {
	set_scanwin_items_status();
	return;
    }

    if (myentry == (struct mem_rec *)NULL)
	not_my_msgs = TRUE;
    else
	not_my_msgs = (strcmp(cur_msg_user, myentry->logid) != 0);

    xv_set(scan_ip->choice_item, PANEL_INACTIVE, not_my_msgs, NULL);

    /*
     *  The "From" header for the message associated with each list row
     *  is stored in a string pointed to by PANEL_CLIENT_DATA; free all
     *  these strings:
     */
    num_msgs = (int)xv_get(scan_ip->list_item, PANEL_LIST_NROWS);
    if (num_msgs > 0)
	for (i = 0 ; i < num_msgs ; i ++)
	    free((char *)xv_get(scan_ip->list_item,
		PANEL_LIST_CLIENT_DATA, i));

    msg_cat = (int)xv_get(scan_ip->choice_item, PANEL_VALUE);

    /*
     *  get_pmsg_headers returns a pointer to a list of structs; each
     *  of these structs contains a pointer to a string we'll display
     *  on the scrolling list, and a pointer to a string we'll store as
     *  the list item's PANEL_CLIENT_DATA.  The display string and the
     *  list of structs itself should be freed immediately after we
     *  put everything on the list; the client data will be freed the
     *  next time the list is redone.
     */
    ph_ptr = get_pmsg_headers(cur_msg_user, msg_cat, &new_size);

    for (i = 0; i < new_size; i++) {
	xv_set(scan_ip->list_item,
	    PANEL_LIST_STRING,		i, ph_ptr[i].desc,
	    PANEL_LIST_CLIENT_DATA,	i, ph_ptr[i].id,
	    NULL);
	free(ph_ptr[i].desc);
    }
    
    if (ph_ptr != NULL) free(ph_ptr);

    /*
     *  See if we have to destroy some list rows:
     *
     */
    if (num_msgs > new_size)
	xv_set(scan_ip->list_item,
	    PANEL_LIST_DELETE_ROWS, new_size, num_msgs - new_size,
	    NULL);
	for ( i = num_msgs - 1; i >= new_size; i--)
	    xv_set(scan_ip->list_item, PANEL_LIST_DELETE, i, NULL);

    num_msgs = new_size;
    num_selected = 0;
    cur_msg = BOGUS;

    pr_msg(scan_ip->frame, FALSE, "");
    if (num_msgs > 1)
	xv_set(scan_list_bar, SCROLLBAR_VIEW_START, 0, NULL);
    select_all_msgs(msg_cat == SCM_CHOICE_NEW);
    set_scanwin_items_status();
    xv_set(scan_ip->list_item, XV_SHOW, TRUE, NULL);
    if (msg_cat == SCM_CHOICE_NEW && smsg_is_up())
	display_message();
    else
	clear_smsg_window();
}

/*
 *  Move messages from one message category to another
 */
void
mv_msgs(dest_cat)
int dest_cat;
{
    int r_count, m_count, smsg;
    char **m_list;

    smsg = (int)xv_get(scan_ip->list_item, PANEL_LIST_FIRST_SELECTED);
    if ( smsg != -1 ) {
	m_list = (char **)malloc(sizeof(char *));
	m_list[0] = (char *)xv_get(scan_ip->list_item,
	    PANEL_LIST_CLIENT_DATA, smsg);
	r_count = 1;
	while ( (smsg = (int)xv_get(scan_ip->list_item,
	    PANEL_LIST_NEXT_SELECTED, smsg)) != -1) {
	    r_count++;
	    m_list = (char **)realloc(m_list, sizeof(char *) * r_count);    
	    m_list[r_count - 1] = (char *)xv_get(scan_ip->list_item,
		PANEL_LIST_CLIENT_DATA, smsg);
	}
	    
	m_count = mv_msg_call(cur_msg_user, msg_cat, dest_cat, r_count, m_list);
	free(m_list);
    }

    scm_get_messages();
    if (m_count == 1)
	pr_msg(scan_ip->frame, FALSE, "One message moved.");
    else if (m_count > 1)
	pr_msg(scan_ip->frame, FALSE, "%d messages moved.", m_count);
}

/*
 *  Wrapper to let the scrolling list menu execute the Select function
 */
void
scsl_select_notify(menu, menu_item)
Menu menu;
Menu_item menu_item;
{
    select_all_msgs(TRUE);
}

/*
 *  Wrapper to let the scrolling list menu execute the Show function
 */
void
scsl_show_notify(menu, menu_item)
Menu menu;
Menu_item menu_item;
{
    display_message();
}

/*
 *  Wrapper to let the scrolling list menu execute the Print function
 */
void
scsl_print_notify(menu, menu_item)
Menu menu;
Menu_item menu_item;
{
    print_messages();
}

/*
 * Menu handler for `list_menu'.
 */
Menu
scan_list_menu_handler(menu, op)
Menu		menu;
Menu_generate	op;
{	
    if (op == MENU_DISPLAY && !scan_list_menu_updated) {
	Menu pr_menu;

	xv_set(menu, MENU_TITLE_ITEM, "Messages", NULL);

	scsl_select_item = (Menu_item)xv_create(NULL, MENUITEM,
	    MENU_STRING,	"Select All Msgs",
	    MENU_NOTIFY_PROC,	scsl_select_notify,
	    MENU_INACTIVE,	(num_msgs == num_selected),
	    NULL);
	xv_set(menu, MENU_APPEND_ITEM, scsl_select_item, NULL);

	scsl_show_item = (Menu_item)xv_create(NULL, MENUITEM,
	    MENU_STRING,	"Show...",
	    MENU_NOTIFY_PROC,	scsl_show_notify,
	    MENU_INACTIVE,	(num_msgs == 0),
	    NULL);
	xv_set(menu, MENU_APPEND_ITEM, scsl_show_item, NULL);

	pr_menu = (Menu)xv_get(scan_ip->move_item, PANEL_ITEM_MENU);
	scsl_move_item = (Menu_item)xv_create(NULL, MENUITEM,
	    MENU_STRING,	"Move",
	    MENU_PULLRIGHT,	pr_menu,
	    MENU_INACTIVE,	(num_msgs == 0),
	    NULL);
	xv_set(menu, MENU_APPEND_ITEM, scsl_move_item, NULL);

	scsl_print_item = (Menu_item)xv_create(NULL, MENUITEM,
	    MENU_STRING,	"Print",
	    MENU_NOTIFY_PROC,	scsl_print_notify,
	    MENU_INACTIVE,	(num_msgs == 0),
	    NULL);
	xv_set(menu, MENU_APPEND_ITEM, scsl_print_item, NULL);

	scan_list_menu_updated = TRUE;
    }
    return menu;
}

/*
 *  Initializes atoms we use locally
 */
void
init_atoms()
{
    TARGETS = XInternAtom(dpy, "TARGETS", False);
    LENGTH = XInternAtom(dpy, "LENGTH", False);
    _SUN_DRAGDROP_DONE = XInternAtom(dpy, "_SUN_DRAGDROP_DONE", False);
}

/*
 *  Respond to drag
 */
static int
drag_conv_proc(sel_owner, sel_type, replyBuff, length, format)
Selection_owner sel_owner;
Atom *sel_type;
Xv_opaque *replyBuff;
unsigned long *length;
int *format;
{
    static Atom targs[3] = { NULL, NULL, NULL };
    static char *drag_buf;
    static int drag_len;

    /*
     *  Initialize list of atoms we will respond to
     */
    if ( TARGETS == NULL ) init_atoms();
    if (targs[0] == NULL ) {
	targs[0] = TARGETS;
	targs[1] = XA_STRING;
	targs[2] = _SUN_DRAGDROP_DONE;
    }

    if ( *sel_type == TARGETS ) {
	*format = 32;
	*length = 4;
	*sel_type = XA_ATOM;
	*replyBuff = (Xv_opaque)targs;
	return TRUE;
    }
    else if ( *sel_type == XA_STRING ) {
	if (drag_buf == CNULL) drag_buf = (char *)copy_messages(&drag_len);
	*format = 8;
	*length = drag_len;
	*replyBuff = (Xv_opaque)drag_buf;
	return TRUE;
    }
    else if ( *sel_type == _SUN_DRAGDROP_DONE ) {
	if (drag_buf != CNULL) {
	    free(drag_buf);
	    drag_buf = CNULL;
	}
	*format = 32;
	*length = 0;
	*replyBuff = NULL;
	*sel_type = XInternAtom(dpy, "NULL", False);
	return TRUE;
    }
    else
    	return FALSE;
}

void
create_scan_window()
{
#ifdef SCALABLE
    int xpos, ypos;
#endif

    scan_ip = scanwin_frame_objects_initialize(NULL, base_frame);

    if (scan_params.state != BOGUS)
	frame_set_rect(scan_ip->frame, &scan_params.rect);

#ifdef SCALABLE
    xv_set(scan_ip->show_item, XV_X, wgrid * 2, XV_Y, wgrid, NULL);

    xpos = wgrid * 3 + (int)xv_get(scan_ip->show_item, XV_WIDTH);
    xv_set(scan_ip->move_item, XV_X, xpos, XV_Y, wgrid, NULL);

    xpos += (int)xv_get(scan_ip->move_item, XV_WIDTH) + wgrid;
    xv_set(scan_ip->print_item, XV_X, xpos, XV_Y, wgrid, NULL);

    xpos += (int)xv_get(scan_ip->print_item, XV_WIDTH) + wgrid;
    xv_set(scan_ip->find_item, XV_X, xpos, XV_Y, wgrid, NULL);

    xpos += (int)xv_get(scan_ip->find_item, XV_WIDTH) + wgrid * 2;
    xv_set(scan_ip->torolo_item, XV_X, xpos, XV_Y, wgrid, NULL);


    ypos = wgrid * 2 + (int)xv_get(scan_ip->torolo_item, XV_HEIGHT);
    xv_set(scan_ip->user_item, XV_X, wgrid * 2, XV_Y, ypos, NULL);
    xv_set(scan_ip->fname_item,
	XV_X,	wgrid * 3 + (int)xv_get(scan_ip->user_item, XV_WIDTH),
	XV_Y,	ypos,
	NULL);

    ypos += wgrid + (int)xv_get(scan_ip->user_item, XV_HEIGHT);
    xv_set(scan_ip->choice_item, XV_X, wgrid * 2, XV_Y, ypos, NULL);

    ypos += wgrid + (int)xv_get(scan_ip->choice_item, XV_HEIGHT);
    xv_set(scan_ip->clear_all_item, XV_X, wgrid * 2, XV_Y, ypos, NULL);

    xpos = wgrid * 3 + (int)xv_get(scan_ip->clear_all_item, XV_WIDTH);
    xv_set(scan_ip->select_all_item, XV_X, xpos, XV_Y, ypos, NULL);

    ypos += wgrid + (int)xv_get(scan_ip->clear_all_item, XV_HEIGHT);
    xv_set(scan_ip->list_item, XV_X, wgrid * 2, XV_Y, ypos, NULL);
    window_fit(scan_ip->panel);

    window_fit(scan_ip->frame);
#endif

    scan_list_bar = (Scrollbar)xv_get(scan_ip->list_item,
	PANEL_LIST_SCROLLBAR);

    xv_set((Menu)xv_get(scan_ip->list_item, PANEL_ITEM_MENU),
	MENU_GEN_PROC, scan_list_menu_handler, NULL);
    xv_set(scan_ip->select_all_item, PANEL_INACTIVE, TRUE, NULL);

    rolo_image = (Server_image)xv_create(NULL, SERVER_IMAGE,
	XV_WIDTH,		48,
	XV_HEIGHT,		48,
	SERVER_IMAGE_BITS,	rolo_bits,
	NULL);
    rolo_cursor = (Xv_Cursor)xv_create(NULL, CURSOR,
	CURSOR_IMAGE,		rolo_image,
	CURSOR_XHOT,		32,
	CURSOR_YHOT,		32,
	NULL);
    mrolo_image = (Server_image)xv_create(NULL, SERVER_IMAGE,
	XV_WIDTH,		48,
	XV_HEIGHT,		48,
	SERVER_IMAGE_BITS,	mrolo_bits,
	NULL);
    mrolo_cursor = (Xv_Cursor)xv_create(NULL, CURSOR,
	CURSOR_IMAGE,		mrolo_image,
	CURSOR_XHOT,		32,
	CURSOR_YHOT,		32,
	NULL);
    r_accept_image = (Server_image)xv_create(NULL, SERVER_IMAGE,
	XV_WIDTH,		48,
	XV_HEIGHT,		48,
	SERVER_IMAGE_BITS,	r_accept_bits,
	NULL);
    r_accept_cursor = (Xv_Cursor)xv_create(NULL, CURSOR,
	CURSOR_IMAGE,		r_accept_image,
	CURSOR_XHOT,		32,
	CURSOR_YHOT,		32,
	NULL);

    xv_set(scan_ip->drop_target1,
	PANEL_DROP_DND,		(Drag_drop)xv_create(scan_ip->panel, DRAGDROP,
				    DND_TYPE,		DND_COPY,
				    SEL_CONVERT_PROC,	drag_conv_proc,
				    DND_CURSOR,		rolo_cursor,
				    DND_ACCEPT_CURSOR,	r_accept_cursor, 
				    NULL),
	NULL);

    scan_window_created = TRUE;
}

void
show_scan_window()
{
    if (!scan_window_created) create_scan_window();

    if ( admin || ((globrec.flags & MB_G_TRUSTED) != 0 && cur_sel != BOGUS))
	xv_set(scan_ip->user_item,
	    PANEL_VALUE,	ulist[cur_sel].logid,
	    PANEL_READ_ONLY,	FALSE,
	    NULL);
    else if ((globrec.flags & MB_G_TRUSTED) == 0) {
	if (myentry == (struct mem_rec *)NULL) {
	    pr_msg(scan_ip->frame, TRUE,
		"%s is not currently on the Message Board.", mylogid);
	    return;
	}
	xv_set(scan_ip->user_item,
	    PANEL_VALUE,	myentry->logid,
	    PANEL_READ_ONLY,	TRUE,
	    NULL);
    }

    xv_set(scan_ip->choice_item,
	PANEL_VALUE,	SCM_CHOICE_DEFAULT,
	NULL);
    scm_get_messages();
    set_scm_fname_item();
    xv_set(scan_ip->frame, XV_SHOW, TRUE, NULL);
}

void
create_find_window()
{
    find_ip = scanwin_find_frame_objects_initialize(NULL, base_frame);
    
    xv_set(find_ip->find_panel,
	PANEL_DEFAULT_ITEM, find_ip->find_fwd_item,
	NULL);

    find_window_created = TRUE;
}

void
show_find_window()
{
    if (!find_window_created) create_find_window();
    xv_set(find_ip->find_frame, XV_SHOW, TRUE, NULL);
}

void
create_copy_window()
{
    copy_ip = scanwin_copy_frame_objects_initialize(NULL, base_frame);
    
    xv_set(copy_ip->copy_panel,
	PANEL_DEFAULT_ITEM, copy_ip->copy_copy_item,
	NULL);

    copy_window_created = TRUE;
}

void
show_copy_window()
{
    if (!copy_window_created) create_copy_window();

    if ( strlen((char *)xv_get(copy_ip->copy_dir_item, PANEL_VALUE)) == 0) {
	char wd[MAXPATHLEN];
	
	getwd(wd);
	xv_set(copy_ip->copy_dir_item, PANEL_VALUE, wd, NULL);
	xv_set(copy_ip->copy_panel, PANEL_CARET_ITEM,
	    copy_ip->copy_file_item, NULL);
    }
    xv_set(copy_ip->copy_frame, XV_SHOW, TRUE, NULL);
}

/****************************************
 *  The following routines handle the initial request to copy messages
 *  to the clipboard, as well as the request from another application
 *  for the clipboard data.
 *
 *  Respond to a request for the contents of the CLIPBOARD selection
 */
static int
clip_conv_proc(sel_owner, sel_type, replyBuff, length, format)
Selection_owner sel_owner;
Atom *sel_type;
Xv_opaque *replyBuff;
unsigned long *length;
int *format;
{
    static Atom targs[3] = { NULL, NULL, NULL };

    /*
     *  Initialize list of atoms we will respond to
     */
    if ( TARGETS == NULL ) init_atoms();
    if (targs[0] == NULL ) {
	targs[0] = TARGETS;
	targs[1] = LENGTH;
	targs[2] = XA_STRING;
    }

    if ( *sel_type == TARGETS ) {
	*format = 32;
	*length = 4;
	*sel_type = XA_ATOM;
	*replyBuff = (Xv_opaque)targs;
	return TRUE;
    }
    else if ( *sel_type == LENGTH ) {
	if (clip_buf == CNULL) clip_buf = (char *)copy_messages(&clip_len);
	*format = 32;
	*length = 1;
	*sel_type = XA_INTEGER;
	*replyBuff = (Xv_opaque)&clip_len;
 	return TRUE;
    }
    else if ( *sel_type == XA_STRING ) {
	if (clip_buf == CNULL) clip_buf = (char *)copy_messages(&clip_len);
	*format = 8;
	*length = clip_len;
	*replyBuff = (Xv_opaque)clip_buf;
	return TRUE;
    }
    else
    	return FALSE;
}

/*
 *  Handle loss of the selection
 */
void
sel_lose_proc(sel)
Selection_owner	sel;
{
    pr_msg(scan_ip->frame, FALSE, "(clipboard relinquished)");

    if ( clip_buf != CNULL ) {
	free(clip_buf);
	clip_buf = CNULL;
    }
}

/*
 *  Become the owner of the CLIPBOARD selection
 */
void
sc_own_clipboard()
{
    static Selection_owner sel_owner = NULL;
    if (sel_owner == NULL)
	sel_owner = xv_create(scan_ip->frame, SELECTION_OWNER,
	    SEL_CONVERT_PROC,	clip_conv_proc,
	    SEL_LOSE_PROC,	sel_lose_proc,
	    NULL);

    if ( clip_buf != CNULL ) {
	free(clip_buf);
	clip_buf = CNULL;
    }

    clip_buf = (char *)copy_messages(&clip_len);

    xv_set(sel_owner,
	SEL_OWN,	TRUE,
	SEL_RANK_NAME,	"CLIPBOARD",
	NULL);
}

/*
 * Notify callback function for `find_item'.
 */
void
sc_find_notify(item, event)
Panel_item	item;
Event		*event;
{
    show_find_window();
}

/*
 * Event callback function for `torolo_item'.
 */
void
sc_torolo_event(item, event)
Panel_item	item;
Event		*event;
{
    if ( event_action(event) == ACTION_SELECT &&
	!(int)xv_get(item, PANEL_INACTIVE)) {
	Xv_Cursor c;
	char *tf;

	if (num_selected > 1)
	    c = mrolo_cursor;
	else
	    c = rolo_cursor;

	tf = tempnam("/tmp", "mb.");
	dnd_messages(tf);
	xview_send_drag_event(base_frame, c, tf);
	look_busy(TRUE);
	sleep(1);
	unlink(tf);
	free(tf);
	look_busy(FALSE);
    }
    else
	panel_default_handle_event(item, event);
}

/*
 * Notify callback function for `user_item'.
 */
Panel_setting
sc_user_notify(item, event)
Panel_item	item;
Event		*event;
{
    set_scm_fname_item();
    xv_set(scan_ip->choice_item, PANEL_VALUE, SCM_CHOICE_DEFAULT, NULL);
    scm_get_messages();
    return PANEL_NONE;		/* Freeze focus on this item */
}

/*
 * Notify callback function for `choice_item'.
 */
void
sc_choice_notify(item, value, event)
Panel_item	item;
int		value;
Event		*event;
{	
    set_scm_fname_item();
    strcpy(cur_msg_user, (char *)xv_get(scan_ip->user_item, PANEL_VALUE));

    /*
     *  If this isn't the current user's messages, don't let them choose any
     *  category other than NEW
     */
    if (myentry == (struct mem_rec *)NULL ||
	(strcmp(cur_msg_user, myentry->logid) != 0 &&
	value != SCM_CHOICE_NEW))
	xv_set(item, PANEL_VALUE, SCM_CHOICE_NEW, NULL);
    scm_get_messages();
}

/*
 * Notify callback function for `copy_file_item'.
 */
Panel_setting
copy_file_notify(item, event)
Panel_item	item;
Event		*event;
{
    if (event_shift_is_down(event))
        return PANEL_PREVIOUS;
    else {
        sc_copy_to_file();
        if ((int)xv_get(copy_ip->copy_frame, FRAME_CMD_PIN_STATE) ==
	    FRAME_CMD_PIN_OUT)
	    xv_set(copy_ip->copy_frame, XV_SHOW, FALSE, NULL);
	return PANEL_NONE;
    }
}

/*
 * Notify callback function for `copy_copy_item'.
 */
void
copy_copy_notify(item, event)
Panel_item	item;
Event		*event;
{
    sc_copy_to_file();
}

/*
 * Menu handler for `move_menu (to Next Category)'.
 */
Menu_item
sc_to_next_handler(item, op)
Menu_item	item;
Menu_generate	op;
{
    switch (op) {
    case MENU_DISPLAY:
	if (scmm_next_item == (Menu_item)NULL)
	    scmm_next_item = item;
	xv_set(item,
	    MENU_INACTIVE, (msg_cat == SCM_MAX_CHOICE) ||
		(not_my_msgs && (msg_cat != SCM_CHOICE_NEW)),
	    NULL);
	break;

    case MENU_NOTIFY:
	if (msg_cat < SCM_MAX_CHOICE)
	    mv_msgs(msg_cat + 1);
	break;
    }
    return item;
}

/*
 * Menu handler for `move_menu (to New)'.
 */
Menu_item
sc_to_new_handler(item, op)
Menu_item	item;
Menu_generate	op;
{
    switch (op) {
    case MENU_DISPLAY:
	if (scmm_new_item == (Menu_item)NULL)
	    scmm_new_item = item;
	xv_set(item, MENU_INACTIVE, not_my_msgs || (msg_cat == SCM_CHOICE_NEW),
	    NULL);
	break;

    case MENU_NOTIFY:
	mv_msgs(SCM_CHOICE_NEW);
	break;
    }
    return item;
}

/*
 * Menu handler for `move_menu (to Pending)'.
 */
Menu_item
sc_to_pending_handler(item, op)
Menu_item	item;
Menu_generate	op;
{
    switch (op) {
    case MENU_DISPLAY:
	if (scmm_pending_item == (Menu_item)NULL)
	    scmm_pending_item = item;
	xv_set(item, MENU_INACTIVE, not_my_msgs || (msg_cat == SCM_CHOICE_PEN),
	    NULL);
	break;

    case MENU_NOTIFY:
	mv_msgs(SCM_CHOICE_PEN);
	break;
    }
    return item;
}

/*
 * Menu handler for `move_menu (to Complete)'.
 */
Menu_item
sc_to_complete_handler(item, op)
Menu_item	item;
Menu_generate	op;
{
    switch (op) {
    case MENU_DISPLAY:
	if (scmm_complete_item == (Menu_item)NULL)
	    scmm_complete_item = item;
	xv_set(item, MENU_INACTIVE, not_my_msgs || msg_cat == (SCM_CHOICE_COM),
	    NULL);
	break;

    case MENU_NOTIFY:
	mv_msgs(SCM_CHOICE_COM);
	break;
    }
    return item;
}

/*
 * Menu handler for `move_menu (to Trash)'.
 */
Menu_item
sc_to_trash_handler(item, op)
Menu_item	item;
Menu_generate	op;
{
    switch (op) {
    case MENU_DISPLAY:
	if (scmm_trash_item == (Menu_item)NULL)
	    scmm_trash_item = item;
	xv_set(item, MENU_INACTIVE, not_my_msgs || msg_cat == (SCM_CHOICE_TRA),
	    NULL);
	break;

    case MENU_NOTIFY:
	mv_msgs(SCM_CHOICE_TRA);
	break;
    }
    return item;
}

/*
 * Menu handler for `move_menu (Copy to Clipboard)'.
 */
Menu_item
sc_to_clip_handler(item, op)
Menu_item	item;
Menu_generate	op;
{
    switch (op) {
    case MENU_DISPLAY:
	if (scmm_clip_item == (Menu_item)NULL)
	    scmm_clip_item = item;
	xv_set(item, MENU_INACTIVE, not_my_msgs, NULL);
	break;

    case MENU_NOTIFY:
	sc_own_clipboard();
	break;
    }
    return item;
}

/*
 * Menu handler for `move_menu (Copy to File...)'.
 */
Menu_item
sc_to_file_handler(item, op)
Menu_item	item;
Menu_generate	op;
{
    switch (op) {
    case MENU_DISPLAY:
	if (scmm_file_item == (Menu_item)NULL)
	    scmm_file_item = item;
	xv_set(item, MENU_INACTIVE, not_my_msgs, NULL);
	break;

    case MENU_NOTIFY:
	show_copy_window();
	break;
    }
    return item;
}

/*
 * Notify callback function for `show_item'.
 */
void
sc_show_notify(item, event)
Panel_item	item;
Event		*event;
{
    display_message();
}

/*
 * Notify callback function for `print_item'.
 */
void
sc_print_notify(item, event)
Panel_item	item;
Event		*event;
{	
    print_messages();
}

/*
 * Notify callback function for `clear_all_item'.
 */
void
sc_clear_all_notify(item, event)
Panel_item	item;
Event		*event;
{
    select_all_msgs(FALSE);
}

/*
 * Notify callback function for `select_all_item'.
 */
void
sc_select_all_notify(item, event)
Panel_item	item;
Event		*event;
{
    select_all_msgs(TRUE);
}

/*
 * Notify callback function for `list_item'.
 */
int
sc_list_notify(item, string, client_data, op, event)
Panel_item	item;
char		*string;
Xv_opaque	client_data;
Panel_list_op	op;
Event		*event;
{
    switch(op) {
    case PANEL_LIST_OP_DESELECT:
	num_selected--;
	set_scanwin_items_status();
	break;
    case PANEL_LIST_OP_SELECT:
	num_selected++;
	set_scanwin_items_status();
	break;
    }
    return XV_OK;
}

/*
 * Done callback function for `frame'.
 */
void
scanwin_done(frame)
Frame frame;
{
    xv_set(frame, XV_SHOW, FALSE, NULL);
    if (find_window_created)
	xv_set(find_ip->find_frame,
	    FRAME_CMD_PIN_STATE,	FRAME_CMD_PIN_OUT,
	    XV_SHOW,			FALSE,
	    NULL);
    if (copy_window_created)
	xv_set(copy_ip->copy_frame,
	    FRAME_CMD_PIN_STATE,	FRAME_CMD_PIN_OUT,
	    XV_SHOW,			FALSE,
	    NULL);
    dismiss_smsg_window();
}

/*
 *  Gathers all the parameters for this window
 */
void
get_scan_params(p)
struct winparams *p;
{
    if (!scan_window_created)
	*p = scan_params;
    else {
	p->state = (int)xv_get(scan_ip->frame, FRAME_CMD_PIN_STATE);
	frame_get_rect(scan_ip->frame, &(p->rect));
    }
}

/*
 * Event callback function for `frame'.
 */
Notify_value
sc_frame_event(win, event, arg, type)
Xv_window	win;
Event		*event;
Notify_arg	arg;
Notify_event_type type;
{
    /*
     *  Check for a resize event
     */
    if (event_action(event) == WIN_RESIZE) {
	int rh, nrows, list_h, xtra;

	/*
	 *  "xtra" is the part of the scrolling list height that is not
	 *  contributed by the rows themselves
	 */
	rh = (int)xv_get(scan_ip->list_item, PANEL_LIST_ROW_HEIGHT);
	xtra = (int)xv_get(scan_ip->list_item, XV_HEIGHT) -
	    rh * (int)xv_get(scan_ip->list_item, PANEL_LIST_DISPLAY_ROWS);

	/*
	 *  Figure out how long the scrolling list should be now
	 */
	list_h = (int)xv_get(scan_ip->panel, XV_HEIGHT) -
	    (int)xv_get(scan_ip->list_item, XV_Y) - wgrid * 2;

	nrows = (list_h - xtra) / rh;
	list_h = nrows * rh  + xtra;

	xv_set(scan_ip->list_item,
	    PANEL_LIST_WIDTH,		(int)xv_get(scan_ip->frame, XV_WIDTH) -
					wgrid * 4,
	    XV_HEIGHT,			list_h,
	    PANEL_LIST_DISPLAY_ROWS,	nrows,
	    NULL);
    }
    return notify_next_event_func(win, (Notify_event) event, arg, type);
}
